// ColorSync routines
//
// David Hayward 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains some useful routines for dealing with profiles
// and picts with embedded profiles. 
// 
// 12/13/94	david	first cut

#include <Resources.h>
#include <Gestalt.h>
#include <QDOffscreen.h>

#include "colorsyncUtils.h"
#include "qdUtils.h"
#include "stringUtils.h"
#include "resourceUtils.h"
#include "myDrawMatchedPict.h"

// Uncomment one of the following:
//#define BUILDING_FOR_CS20_ONLY		1
//#define BUILDING_FOR_CS21_ONLY		1
//#define BUILDING_FOR_CS25_ONLY		1
//#define BUILDING_FOR_CS21_AND_LESS	1
#define BUILDING_FOR_CS25_AND_LESS	1


#if defined(BUILDING_FOR_CS20_ONLY)
	#define FOR_CS20			1
	#define FOR_CS21			0
	#define FOR_CS25			0
#elif defined(BUILDING_FOR_CS21_ONLY)
	#define FOR_CS20			0
	#define FOR_CS21			1
	#define FOR_CS25			0
#elif defined(BUILDING_FOR_CS25_ONLY)
	#define FOR_CS20			0
	#define FOR_CS21			0
	#define FOR_CS25			1
#elif defined(BUILDING_FOR_CS21_AND_LESS)
	#define FOR_CS20			1
	#define FOR_CS21			1
	#define FOR_CS25			0
#elif defined(BUILDING_FOR_CS25_AND_LESS)
	#define FOR_CS20			1
	#define FOR_CS21			1
	#define FOR_CS25			1
#else // default is BUILDING_FOR_CS20_ONLY
	#define FOR_CS20			1
	#define FOR_CS21			0
	#define FOR_CS25			0
#endif


#define MIN(x,y)	( ((x)<(y)) ? (x) : (y) )
#define MAX(x,y)	( ((x)>(y)) ? (x) : (y) )


/**\
|**| ==============================================================================
|**| PUBLIC GLOBALS
|**| ==============================================================================
\**/
Boolean gCS2_available = false;


/**\
|**| ==============================================================================
|**| PRIVATE GLOBALS
|**| ==============================================================================
\**/
static unsigned long gCount ;
static unsigned long gIndex ;
static unsigned long gHow ;
static Handle gHandle ;


/**\
|**| ==============================================================================
|**| PRIVATE TYPEDEFS
|**| ==============================================================================
\**/
enum {
	cmProfIdentTag					= 'uqid'
};

struct CMProfIdentType {
	OSType 							typeDescriptor;				/* 'dtim' */
	unsigned long 					reserved;
	CMProfileIdentifier 			ident;						/* Variable size */
};
typedef struct CMProfIdentType CMProfIdentType;


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
pascal void		CountProfilesCommentProc	( short kind, short dataSize, Handle dataHandle ) ;
pascal void		GetIndexedHandleCommentProc ( short kind, short dataSize, Handle dataHandle ) ;

pascal void		TextProc		( short byteCount, Ptr textAddr, Point numer, Point denom ) ;
pascal void		LineProc		( Point endPoint ) ;
pascal void		RectProc		( GrafVerb verb, Rect *r ) ;
pascal void		RRectProc		( GrafVerb verb, Rect *r, short ovalWidth, short ovalHeight ) ;
pascal void		OvalProc		( GrafVerb verb, Rect *r ) ;
pascal void		ArcProc			( GrafVerb verb, Rect *r, short startAngle, short arcAngle ) ;
pascal void		PolyProc		( GrafVerb verb, PolyHandle poly ) ;
pascal void		RgnProc			( GrafVerb verb, RgnHandle rgn ) ;
pascal void		BitsProc		( BitMap *bitPtr, Rect *srcRect, Rect *dstRect, short mode, RgnHandle maskRgn ) ;
pascal void		TxMeasProc		( short byteCount, Ptr textAddr, Point *numer, Point *denom, FontInfo *info ) ;



/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	CS2_available
 *------------------------------------------------------------------------------*
 		result	true if colorSync 2.0 or greater is installed, otherwise false.
 	
 	Description:
		Sets the global gCS2_available based on the return value.
\*------------------------------------------------------------------------------*/
Boolean CS2_available ( void ) 
{
	long	result;

 	gCS2_available = false ;

	if (Gestalt(gestaltColorMatchingVersion, &result) == noErr)
   		if ( result >= gestaltColorSync20 )
#if GENERATINGPOWERPC
			if (Gestalt(gestaltColorMatchingAttr, &result) == noErr)
		   		if ( result & (1<<gestaltColorMatchingLibLoaded) )
#endif
 			gCS2_available = true ;

	return gCS2_available ;
}


/*------------------------------------------------------------------------------*\
	CS_version
 *------------------------------------------------------------------------------*
 		result	version of ColorSync installed.  Return 0 if any arry occurs.
\*------------------------------------------------------------------------------*/
long CS_version ( void ) 
{
	long	version = nil;
	OSErr	err;

	err =Gestalt(gestaltColorMatchingVersion, &version);
	if (err) version = nil;

	return version ;
}


/*------------------------------------------------------------------------------*\
	OpenProfileFSSpec
 *------------------------------------------------------------------------------*
	given an FSSpec, open the profile and return a CMProfileRef

		spec	(In) A valid FSSpec of a version 1 or 2 profile file.

		prof	(out) If result is noErr, then returns the CMProfileRef of the
				opened profile file, otherwise undefined.
		
 		result	An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError OpenProfileFSSpec ( FSSpec spec, CMProfileRef *prof ) 
{	
	CMError					err = noErr ;
	CMProfileLocation		profLoc ;
	
	profLoc.locType = cmFileBasedProfile;
	profLoc.u.fileLoc.spec = spec;
	err = CMOpenProfile(prof, &profLoc) ;
	return err ;	
}



/*------------------------------------------------------------------------------*\
	ReOpenProfileRef
 *------------------------------------------------------------------------------*
	given an exiting CMProfileRef, open a new reference to the profile

		newProf	(out) If result is noErr, then returns a new reference to the
				profile, otherwise undefined.  If the location of the profile
				is of type cmNoProfileBase, then make a copy of the profile
				into a temperary file, and return a reference to it.

		prof	(in) The CMProfileRef to reopen
		
 		result	An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError ReOpenProfileRef ( CMProfileRef *newProf, CMProfileRef prof ) 
{	
	CMError					err = noErr ;
	
	if ( prof==nil )
	{
		*newProf = nil ;
		return noErr ;
	}
	
#if FOR_CS21 || FOR_CS25
#if FOR_CS20
	if ( CS_version() >= 0x0210 )
#endif
	{
		err = CMCloneProfileRef(prof);
		*newProf = prof ;
		return err ;
	}
#endif
			
#if FOR_CS20
	{
		CMProfileLocation	profLoc ;
		
		// get location of profile
		err = CMGetProfileLocation( prof, &profLoc) ;
		if (err) return err ;
	
	switch (profLoc.locType)
	{
		case  cmNoProfileBase :
				err = CMCopyProfile( newProf, nil, prof) ;
			break ;
			
		case cmFileBasedProfile :
				err = CMOpenProfile( newProf, &profLoc ) ;
			break ;
	
		default :
				err = 1 ; // should have a better error code
				break ;
		}
	}
#endif
	
	return err ;	
}


/*------------------------------------------------------------------------------*\
	GetProfileVersion
 *------------------------------------------------------------------------------*
		prof	(in) the CMProfileRef to examine
		
		vers	(out) if result is noErr, then returns the version field of the
				profile's header,  otherwise undefined.
				
 		result	an error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError GetProfVersion ( CMProfileRef prof, unsigned long *vers)
{
	CMError					err = noErr ;
	CMAppleProfileHeader	head ;

	err = CMGetProfileHeader(prof, &head) ;
	if (err) return err ;

	*vers = head.cm2.profileVersion ;
	
	return err ;
}


/*------------------------------------------------------------------------------*\
	GetProfileIdentifierSize
 *------------------------------------------------------------------------------*
		ident	(in) the CMProfileIdentifierPtr to examine
		
		size	(out) if result is noErr, then returns the size of the 
				CMProfileIdentifierPtr,  otherwise undefined.
				
 		result	an error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError GetProfileIdentifierSize ( CMProfileIdentifierPtr ident, unsigned long *size )
{
	*size = sizeof(CM2Header) + 
			sizeof(CMDateTime) + 
			sizeof(unsigned long) + 
			ident->ASCIIProfileDescriptionLen ;
	return noErr ;
}


/*------------------------------------------------------------------------------*\
	GetProfileName
 *------------------------------------------------------------------------------*
		prof	(in) The CMProfileRef to examine
		
		name	(out) If result is noErr, then returns the name string of the
				profile reference, otherwise undefined.  If the profile reference
				is nil the string in resource 'STR ' 128 is returned.

 		result	An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError GetProfName ( CMProfileRef prof, StringPtr name )
{
	CMError					err = noErr ;
	ScriptCode				code ;
	
	if (prof==nil)
		GetStringPtr ( 128, name ) ;
	else
		err = CMGetScriptProfileDescription( prof, name, &code) ;
	
	return err ;
}


/*------------------------------------------------------------------------------*\
	GetProfileIdentName
 *------------------------------------------------------------------------------*
		ident	(in) The CMProfileIdentifierPtr to examine
		
		name	(out) If result is noErr, then returns the name string of the
				profile reference, otherwise undefined.

 		result	An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError GetProfileIdentName ( CMProfileIdentifierPtr ident, StringPtr name )
{
	unsigned long	size ;
	
	size = ident->ASCIIProfileDescriptionLen - 1;
	if (size>255) size=255;
	name[0] = size ;
	BlockMoveData( (void*)&(ident->ASCIIProfileDescription), (void*)&(name[1]), size);
	return noErr ;
}


/*------------------------------------------------------------------------------*\
	ProfileIdentComment
 *------------------------------------------------------------------------------*
 	Sends a CMProfileIdentifierPtr picComment to the current port.
\*------------------------------------------------------------------------------*/
CMError ProfileIdentComment ( CMProfileIdentifierPtr ident )
{
	CMError					err = noErr ;
	unsigned long			size ;
	Handle					h ;
	
	err = GetProfileIdentifierSize( ident, &size ) ;
	if (err) return err ;
	
	h = NewHandle( size+4 ) ;
	if (!h) return MemError() ;
	
	HLock( h );
	
	*((long *)(*h)) = cmProfileIdentifierSel ;
	BlockMoveData( (Ptr)ident,(Ptr)(*h + 4),size);
	
	HUnlock( h );
	
	PicComment( cmComment, size+4, h);
	DisposeHandle( h );
	return noErr;
}


/*------------------------------------------------------------------------------*\
	EndProfileComment
 *------------------------------------------------------------------------------*
 	Sends a cmEndProfile picComment to the current port.
\*------------------------------------------------------------------------------*/
void EndProfileComment ( void )
{
	PicComment( cmEndProfile, 0, 0 ) ;
}


/*------------------------------------------------------------------------------*\
	PrependProfileToPicHandle
 *------------------------------------------------------------------------------*
 	Create a new picture which has an embedded profile at the begining.
 	
		pict		(in) PicHandle to prepend the profile to
		
		pictNew		(out) if result is noErr, then returns a new color
					picture which includes the profile, otherwise undefined.

		prof		(in) a profile reference to  embeded profile.
									
 		result		An error code if any error occured, otherwise noErr.
 		
\*------------------------------------------------------------------------------*/
CMError PrependProfileToPicHandle ( PicHandle pict,
									PicHandle *pictNew,
									CMProfileRef prof,
									unsigned long flags )
{
	CMError					err = noErr ;
	CGrafPtr				savePort;
	GDHandle				saveGDev;
	GWorldPtr				smallOff;
	Rect					pictRect;
	
	// create a temporary gworld
	err = NewSmallGWorld( &smallOff ) ;
	if (err) return err ;
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( smallOff, nil ) ;
	pictRect = (**pict).picFrame ;
	ClipRect( &pictRect ) ;						// Important: set clipRgn
	
	// create a new picture
	*pictNew = OpenPicture( &pictRect ) ;		// start recording
	
	err = NCMUseProfileCommentCompat( prof, flags) ;
	
	DrawPicture( pict, &pictRect ) ;			// the original picture
	EndProfileComment() ;						// be sure to to off the prepended profile
	ClosePicture() ;							// stop recording

	if (err) KillPicture( *pictNew ) ;			// can't embed a CS1 profile
	
	SetGWorld( savePort, saveGDev ) ;
	DisposeGWorld( smallOff ) ;
	
	return err;
}


/*------------------------------------------------------------------------------*\
	PrependProfileIdentToPicHandle
 *------------------------------------------------------------------------------*
 	Create a new picture which has an embedded profile at the begining.
 	
		pict		(in) PicHandle to prepend the profile to
		
		pictNew		(out) if result is noErr, then returns a new color
					picture which includes the profile, otherwise undefined.

		prof		(in) a profile reference to  embeded profile.
									
 		result		An error code if any error occured, otherwise noErr.
 		
\*------------------------------------------------------------------------------*/
CMError PrependProfileIdentToPicHandle ( PicHandle pict,
									PicHandle *pictNew,
									CMProfileIdentifierPtr ident )
{
	CMError					err = noErr ;
	CGrafPtr				savePort;
	GDHandle				saveGDev;
	GWorldPtr				smallOff;
	Rect					pictRect;
	
	// create a temporary gworld
	err = NewSmallGWorld( &smallOff ) ;
	if (err) return err ;
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( smallOff, nil ) ;
	pictRect = (**pict).picFrame ;
	ClipRect( &pictRect ) ;						// Important: set clipRgn
	
	// create a new picture
	*pictNew = OpenPicture( &pictRect ) ;		// start recording
	err = ProfileIdentComment( ident ) ;		// embed the ProfIdent
	DrawPicture( pict, &pictRect ) ;			// the original picture
	EndProfileComment() ;						// be sure to to off the prepended profile
	ClosePicture() ;							// stop recording

	if (err) KillPicture( *pictNew ) ;			// failed to embed profile id
	
	SetGWorld( savePort, saveGDev ) ;
	DisposeGWorld( smallOff ) ;
	
	return err;
}


typedef struct IterateCompat
{
	CMProfileIterateUPP	proc;
	OSErr					osErr;
	void*					refCon;
} IterateCompatRec, *IterateCompatPtr;

static pascal Boolean ProfileSearchFilter (CMProfileRef prof, void *refCon)
{
	OSErr					err = noErr;
	IterateCompatPtr		refConCompatPtr;
	CMProfileIterateData	iterData;
	
	// Set up refcon
	refConCompatPtr = (IterateCompatPtr)refCon;
	
	// if we had an error from an earlier profile, bail
	err = refConCompatPtr->osErr;
	if (err) goto bail;
	
	// Try to get the profile's location
	err = CMGetProfileLocation(prof, &iterData.location);
	if (err) goto bail;
	
	// Try to get the profile's header
	err = CMGetProfileHeader(prof, (CMAppleProfileHeader*)&iterData.header);
	if (err) goto bail;
	
	// Try to get the profile's name
	err = CMGetScriptProfileDescription(prof, iterData.name, &iterData.code);
	if (err) goto bail;
	
	iterData.dataVersion = cmProfileIterateDataVersion1;
	
	// Call the itterate callback
	err = CallCMProfileIterateProc( refConCompatPtr->proc, &iterData, refConCompatPtr->refCon);
	if (err) goto bail;
	
bail:
	
	if (err)
		refConCompatPtr->osErr = err;
	
	return true;   // exclude the profile;
}

CMError CMIterateColorSyncFolderCompat (CMProfileIterateUPP proc,
										 unsigned long *seed, unsigned long *count,
										 void *refCon)
{
	CMError					err = noErr ;

#if FOR_CS25
#if FOR_CS20 || FOR_CS21
	if ( CS_version() >= 0x0250 )
#endif
		return CMIterateColorSyncFolder( proc, seed, count, refCon);
#endif
	
#if FOR_CS20 || FOR_CS21
	{
		CMProfileSearchRef	searchResult;
		CMSearchRecord		searchSpec;
		unsigned long 		count;
		IterateCompatRec	refConCompat;
		
		searchSpec.filter = NewCMProfileFilterProc(ProfileSearchFilter);
		searchSpec.searchMask = cmMatchAnyProfile;
		
		refConCompat.proc = proc;
		refConCompat.osErr = noErr;
		refConCompat.refCon = refCon;
		
		err = CMNewProfileSearch( &searchSpec, (void*)&refConCompat, &count, &searchResult);
		if (err==noErr)
		{
			CMDisposeProfileSearch(searchResult);
			err = refConCompat.osErr;
		}
		DisposeRoutineDescriptor(searchSpec.filter);
	}
#endif

	return err;
}


/*------------------------------------------------------------------------------*\
	NCMUseProfileCommentCompat
 *------------------------------------------------------------------------------*


The issue of embedding CS1 profiles in my CSDemo application is unrelated but
important nonetheless.  The problem is that the NCMUseProfileComment() routine
does not work correctly when embedding CS1 profile.  The call does not return an
error but the results are incorrect.  Here's why:

NCMUseProfileComment is designed to embed CS2 profiles with the cmComment
picComment.  In contrast,  CMUseProfileComment() is designed to embed CS1
profiles with the cmBeginProfile picComment.  Now, when you open CS1 profile
using the CS2 API, you are returned a CMProfileRef - not a CMProfileHandle.  On
opening the CS1 profile, ColorSync "converts" it so that its contents can be
accessed using the CS2 API.  ColorSync  knows how to make use of the "converted"
profile but it isnot a valid CS2 profile in the conventional sense.

Here's where things get messy.  If you call  NCMUseProfileComment() for this
ProfileRef, then ColorSync tries to embed the quasi-ICC profile with the
cmComment picComment.  I doubt that this profile will work if it is extracted
later and there is the added problem the embedded profile will be ignores
entirely if the PICT is drawn on a system the CS1 because the profile was not
embedded with the cmBeginProfile  picComment.

Technically this is a bug in NCMUseProfileComment().  Ideally it should embed the
old-fashioned way if it is passed a CS1 ProfileRef.  This bug is fixed in the 2.1 
release of ColorSync but if your want your application to also work under 
ColorSync 2.0 you should use this routine. 

A less critical bug in the ColorSync 2.0 implimentation of NCMUseProfileComment()
is that it doest not cmEmbeddedProfile bit before embedding the profile. Again, 
this bug is fixed in the 2.1 release of ColorSync but if your want your application 
to work correctly under ColorSync 2.0 you should use this routine.

Also, the ColorSync 2.1 implimentation of NCMUseProfileComment() has a new feature.
If you pass in cmEmbedProfileIdentifier for the flags parameter, then it will
embed the smaller CMProfileIdentifier into the pict instead of the whole profile.
If you want your want your application th use this featue and to work correctly 
under ColorSync 2.0 you should use this routine.
\*------------------------------------------------------------------------------*/

CMError NCMUseProfileCommentCompat(CMProfileRef prof, unsigned long flags)
{
	OSErr					err = noErr ;

	if (prof==nil) return paramErr ;
	
#if FOR_CS21 || FOR_CS25
#if FOR_CS20
	if ( CS_version() >= 0x0210 )
#endif
		return NCMUseProfileComment(prof,flags) ;	// embed the profile
#endif
	
#if FOR_CS20
	{
		CMAppleProfileHeader	head ;
		unsigned long			vers ;
		
		// get the profile header
		err = CMGetProfileHeader(prof, &head) ;
		if (err) return err ;
		
		// get the version from profile header
		vers = head.cm2.profileVersion ;
		
		if (vers == cmCS1ProfileVersion)			// if version 1 profile...
		{
			CMProfileHandle			v1prof;
			
			err = ConvertV1Profile( prof,&v1prof);	// Convert v1 ProfileRef to CMProfileHandle 
			if (err) return err ;
			
			err = CMUseProfileComment(v1prof);	// embed the CS1 profile
		}
		else if (vers >= cmCS2ProfileVersion)		// if version 2 profile...
		{
			if (flags == cmEmbedWholeProfile )
			{
				unsigned long			saveflags ;
				
				// set the embeded profile flag bit
				saveflags = head.cm2.flags ;
				head.cm2.flags |= (1<<cmEmbeddedProfile) ;
				err = CMSetProfileHeader(prof, &head) ;
				if (err) return err ;
		
				err = NCMUseProfileComment(prof,nil) ;	// embed the CS2 profile
				if (err) return err ;
				
				// restore the original flag just in case
				head.cm2.flags = saveflags ;
				CMSetProfileHeader(prof, &head) ;
			}
			else
			{
				CMProfileIdentifierPtr	ident;
				unsigned long			size;
				
				err = CMCreateProfileIdentifierCompat( prof, nil, &size);
				if (err) return err ;
		
				ident = (CMProfileIdentifierPtr)NewPtrClear(size);
				if (!ident) err = MemError();
				if (err) return err ;
				
				err = CMCreateProfileIdentifierCompat( prof, ident, &size );
				if (!err)
					err = ProfileIdentComment( ident ) ;	// embed the ProfIdent
				
				DisposePtr( (Ptr)ident );
			}
		}				
	}
#endif
	
	return err ;
}


CMError CMCreateProfileIdentifierCompat(CMProfileRef prof, CMProfileIdentifierPtr ident, unsigned long *size)
{
	CMError					err = noErr ;

#if FOR_CS21 || FOR_CS25
#if FOR_CS20
	if ( CS_version() >= 0x0210 )
#endif
		return CMCreateProfileIdentifier( prof, ident, size) ;
#endif

#if FOR_CS20
	{
		unsigned long			calibsize, asciisize, asciisizeSize = 4 ;
		
		/* first-off, get the size of the ascii string in the description tag */
		/* no matter what, we'll need this info */
		err = CMGetPartialProfileElement( prof, cmProfileDescriptionTag, 
										  8, &asciisizeSize, &asciisize) ;
		if (err) return err;

		if (ident==nil)
		{
			if (size==nil) return paramErr ;
			*size = sizeof(CM2Header) + sizeof(CMDateTime) + sizeof(long) + asciisize ;
		}
		else
		{
			/* get the header first */
			err = CMGetProfileHeader(prof, (CMAppleProfileHeader*)&(ident->profileHeader));
			if (err) return err;
			
			/* set the calibrationDate tag */
			calibsize = sizeof(CMDateTime);
			err = CMGetPartialProfileElement( prof, cmCalibrationDateTimeTag, 
											  8, &calibsize, 
											  &(ident->calibrationDate) ) ;
			if (err) {
				/* set the calibration date to 0s */
				ident->calibrationDate.year = 0;
				ident->calibrationDate.month = 0;
				ident->calibrationDate.dayOfTheMonth = 0;
				ident->calibrationDate.hours = 0;
				ident->calibrationDate.minutes = 0;
				ident->calibrationDate.seconds = 0;
				err = noErr;
			}
			
			ident->ASCIIProfileDescriptionLen = asciisize;
			
			/* get the size of the ascii string in the description tag */
			err = CMGetPartialProfileElement( prof, cmProfileDescriptionTag, 
											  12, &asciisize, 
											  &(ident->ASCIIProfileDescription) ) ;
		}
	}
#endif

	return err ;
}


CMError ConvertV1Profile( CMProfileRef prof, CMProfileHandle *h )
{
	CMError				err = noErr ;
	
#if FOR_CS21 || FOR_CS25
#if FOR_CS20
	if ( CS_version() >= 0x0210 )
#endif
		return CMConvertProfile2to1( prof, h );
#endif

#if FOR_CS20
	{
		CMHeader			head ;
		unsigned long		vers ;
		UInt32				headSize;
		UInt32				chrmSize;
		UInt32				trc_Size;
		UInt32				nameSize;
		UInt32				custSize;
		UInt32				data;
		
		if ( !prof || !h ) return paramErr ;
		
		// get the profile header
		err = CMGetProfileHeader(prof, (CMAppleProfileHeader*)&head) ;
		if (err) return err ;
		headSize = sizeof(CMHeader);
	
		// get the version from profile header
		vers = head.applProfileVersion ;
		if (vers != cmCS1ProfileVersion) return cmInvalidProfile ;		// if version 1 profile...
		
		// get sized of other parts
		if (!err) err = CMGetProfileElement( prof, 'chrm', &chrmSize, nil) ;
		if (!err) err = CMGetProfileElement( prof, 'trc ', &trc_Size, nil) ;
		if (!err) err = CMGetProfileElement( prof, 'name', &nameSize, nil) ;
		if (!err) err = CMGetProfileElement( prof, 'cust', &custSize, nil) ;
		if (err) return err ;
		
		// allocate handle
		*h = (CMProfileHandle)NewHandle( headSize+chrmSize+trc_Size+nameSize+custSize );
		if (*h == nil) return MemError();
		HLock( (Handle)*h );
	
		// copy the header over to the handle
		data = (UInt32) **h;
		
		BlockMoveData( (void*)&head, (void*)data, headSize);			data+=headSize ;
		if (!err) err = CMGetProfileElement( prof, 'chrm', nil, (void*)data) ;	data+=chrmSize ;
		if (!err) err = CMGetProfileElement( prof, 'trc ', nil, (void*)data) ;	data+=trc_Size ;
		if (!err) err = CMGetProfileElement( prof, 'name', nil, (void*)data) ;	data+=nameSize ;
		if (!err) err = CMGetProfileElement( prof, 'cust', nil, (void*)data) ;	data+=custSize ;
		if (err) DisposeHandle( (Handle)*h ) ;
	}
#endif

	return err;
}

/*------------------------------------------------------------------------------*\
	ProofPicHandle
 *------------------------------------------------------------------------------*
 	Create a new picture which is soft-proofed version of the input picture.
 	Create a new picture which soft-proofed according to the embeded profiles (if
 	any) and the supplied profiles of the destination and preview device (i.e.
 	the printer and monitor profiles) 
 	This routine calls myDrawProofedPicture() which is in myDrawMatchedPicture.c.
 	
		pict		(in) PicHandle to ckeck
		
		pictProofed	(out) if result is noErr, then returns a new color
					picture which is a matched version of the input picture,
					otherwise undefined.

		srceProf	(in) a profile reference to use when no embeded profile is 
					specified in the pict.  Typically this would be nil (i.e.
					the system profile).
		
		destProf	(in) a profile reference for the destination device.
					Typically this would be profile for the output devive
		
		prevProf	(in) a profile reference for the preview device. 
					Typically this would be profile for the display devive
					
		ticks		(out) ptr to a long, to which will be stored the duration
					in ticks (1/60 sec) of the proofing operation, discounting i/o.
							
		pixels		(out) ptr to a long, to which will be stored the number of
					pixels operated on by ColorSync.
							
 		result		An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError ProofPicHandle ( PicHandle pict,
						 PicHandle *pictProofed,
						 CMProfileRef srceProf,
						 CMProfileRef destProf,
						 CMProfileRef prevProf,
						 unsigned long *ticks,
						 unsigned long *pixels )
{
	OSErr			err = noErr ;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	GWorldPtr		smallOff;
	Rect			pictRect;
	
	pictRect = (**pict).picFrame ;

	err = NewSmallGWorld( &smallOff ) ;
	if (err) return err ;
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( smallOff, nil ) ;
	
	ClipRect( &pictRect ) ;						// Important: set clipRgn
	
	*pictProofed = OpenPicture( &pictRect ) ;	// start recording
	err = myDrawProofedPicture( pict, srceProf, destProf, prevProf, &pictRect, ticks, pixels ) ;
	ClosePicture() ;			// stop recording

	if (err)
	{
		DisposeHandle( (Handle)*pictProofed ) ;
		*pictProofed = nil ;
	}
	
	SetGWorld( savePort, saveGDev ) ;
	DisposeGWorld( smallOff ) ;

	return err ;
}


/*------------------------------------------------------------------------------*\
	MatchPicHandle
 *------------------------------------------------------------------------------*
 	Create a new picture which matched according to the embeded profiles (if any)
 	and the supplied profile of the preview device (i.e. the monitor profile) 
 	This routine calls myDrawMatchedPicture() which is in myDrawMatchedPicture.c.
 	
		pict		(in) PicHandle to match
		
		pictMatched	(out) if result is noErr, then returns a new color
					picture which is black wherever the input pict is out of
					gamut, otherwise undefined.

		srceProf	(in) a profile reference to use when no embeded profile is 
					specified in the pict.  Typically this would be nil (i.e.
					the system profile).
		
		prevProf	(in) a profile reference for the preview device. 
					Typically this would be profile for the display devive
							
		ticks		(out) ptr to a long, to which will be stored the duration
					in ticks (1/60 sec) of the matching operation, discounting i/o.
							
		pixels		(out) ptr to a long, to which will be stored the number of
					pixels operated on by ColorSync.
							
 		result		An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError MatchPicHandle ( PicHandle pict,
						 PicHandle *pictMatched,
						 CMProfileRef srceProf,
						 CMProfileRef prevProf,
						 unsigned long *ticks,
						 unsigned long *pixels )
{
	OSErr			err = noErr ;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	GWorldPtr		smallOff;
	Rect			pictRect;
	PixMapHandle	pixmap;
	
	pictRect = (**pict).picFrame ;

	err = NewSmallGWorld( &smallOff ) ;
	if (err) return err ;
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( smallOff, nil ) ;
	
	ClipRect( &pictRect ) ;						// Important: set clipRgn
		
	pixmap = GetGWorldPixMap(smallOff);
	*pictMatched = OpenPicture( &pictRect ) ;	// start recording
	if(LockPixels(pixmap))
		err = myDrawMatchedPicture( pict, srceProf, prevProf, &pictRect, ticks, pixels ) ;
	ClosePicture() ;							// stop recording
	UnlockPixels(pixmap);
	
	if (err)
	{
		DisposeHandle( (Handle)*pictMatched ) ;
		*pictMatched = nil ;
	}

	SetGWorld( savePort, saveGDev ) ;
	DisposeGWorld( smallOff ) ;
	
	return err ;
}


/*------------------------------------------------------------------------------*\
	CheckPicHandle
 *------------------------------------------------------------------------------*
 	Create a new picture which is black wherever the input picture is out of gamut.
 	This routine calls myDrawCheckedPicture() which is in myDrawMatchedPicture.c.
 	
		pict		(in) PicHandle to ckeck
		
		pictChecked	(out) if result is noErr, then returns a new black & white
					picture which is black wherever the input pict is out of
					gamut, otherwise undefined.

		srceProf	(in) a profile reference to use when no embeded profile is 
					specified in the pict.  Typically this would be nil (i.e.
					the system profile).
		
		destProf	(in) a profile reference for the destination device.
					Typically this would be profile for the output devive
		
		ticks		(out) ptr to a long, to which will be stored the duration
					in ticks (1/60 sec) of the checking operation, discounting i/o.
							
		pixels		(out) ptr to a long, to which will be stored the number of
					pixels operated on by ColorSync.
							
 		result		An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError CheckPicHandle ( PicHandle pict,
						 PicHandle *pictChecked,
						 CMProfileRef srceProf,
						 CMProfileRef destProf,
						 unsigned long *ticks,
						 unsigned long *pixels )
{
	OSErr			err = noErr ;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	GWorldPtr		smallOff;
	Rect			pictRect;
	
	pictRect = (**pict).picFrame ;

	err = NewSmallGWorld( &smallOff ) ;
	if (err) return err ;
	
	GetGWorld( &savePort, &saveGDev ) ;
	SetGWorld( smallOff, nil ) ;
	
	ClipRect( &pictRect ) ;						// Important: set clipRgn
		
	*pictChecked = OpenPicture( &pictRect ) ;	// start recording
	err = myDrawCheckedPicture( pict, srceProf, destProf, &pictRect, ticks, pixels ) ;
	ClosePicture() ;								// stop recording

	if (err)
	{
		DisposeHandle( (Handle)*pictChecked ) ;
		*pictChecked = nil ;
	}

	SetGWorld( savePort, saveGDev ) ;
	DisposeGWorld( smallOff ) ;
	
	return err ;
}


/**\
|**| ==============================================================================
|**| CountProfilesInPicHandle ROUTINES
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	CountProfilesInPicHandle
 *------------------------------------------------------------------------------*
 	count the number of profiles embedded in a PicHandle.
 	
		pict	(in) PicHandle to count the profile in
		
		count	(out) if result is noErr, then returns a one-based count of the
				profiles in the PicHandle, otherwise undefined.

 		result	An error code if any error occured, otherwise noErr.
\*------------------------------------------------------------------------------*/
CMError CountProfilesInPicHandle ( PicHandle pict, unsigned long *count )
{
	OSErr			err = noErr;
	CQDProcs		procs;
		
	// setup bottleneck for piccomments so we can count profiles
	SetStdCProcs(&procs) ;
	procs.textProc    = NewQDTextProc   ( TextProc    ) ;
	procs.lineProc    = NewQDLineProc   ( LineProc    ) ;
	procs.rectProc    = NewQDRectProc   ( RectProc    ) ;
	procs.rRectProc   = NewQDRRectProc  ( RRectProc   ) ;
	procs.ovalProc    = NewQDOvalProc   ( OvalProc    ) ;
	procs.arcProc     = NewQDArcProc    ( ArcProc     ) ;
	procs.polyProc    = NewQDPolyProc   ( PolyProc    ) ;
	procs.rgnProc     = NewQDRgnProc    ( RgnProc     ) ;
	procs.bitsProc    = NewQDBitsProc   ( BitsProc    ) ;
	procs.commentProc = NewQDCommentProc( CountProfilesCommentProc ) ;
	procs.txMeasProc  = NewQDTxMeasProc ( TxMeasProc  ) ;

	// initialize global which is incrimented by the commentProc
	gCount = 0 ;
	
	// draw picture to count profiles
	err = DrawPicHandleUsingBottlenecks ( pict, procs, nil ) ;
	
	// get the result from the global
	*count = gCount ;
		
	// clean up and return
	DisposeRoutineDescriptor( procs.textProc  ) ;
	DisposeRoutineDescriptor( procs.lineProc  ) ;
	DisposeRoutineDescriptor( procs.rectProc  ) ;
	DisposeRoutineDescriptor( procs.rRectProc ) ;
	DisposeRoutineDescriptor( procs.ovalProc  ) ;
	DisposeRoutineDescriptor( procs.arcProc   ) ;
	DisposeRoutineDescriptor( procs.polyProc  ) ;
	DisposeRoutineDescriptor( procs.rgnProc   ) ;
	DisposeRoutineDescriptor( procs.bitsProc  ) ;
	DisposeRoutineDescriptor( procs.commentProc ) ;
	DisposeRoutineDescriptor( procs.txMeasProc  ) ;
	return err;
}

pascal void CountProfilesCommentProc (short kind, short dataSize, Handle dataHandle)
{
	long selector ;

	switch (kind)
	{
		case cmBeginProfile:
			gCount ++ ;								// we found a version 1 profile
			break ;

		case cmComment:
			if ( dataSize <= 4 ) break;				// dataSize too small for selector so break
			selector = *((long *)(*dataHandle)) ;	// get the selector from 1st long in data
			if ( selector == cmBeginProfileSel )
				gCount ++ ;							// we found a version 2 profile
			if ( selector == cmProfileIdentifierSel )
				gCount ++ ;							// we found a ProfileIdentifier
			break ;
	}		
}


/**\
|**| ==============================================================================
|**| GetIndexedProfileFromPicHandle ROUTINES
|**| ==============================================================================
\**/




/*------------------------------------------------------------------------------*\
	GetIndexedProfileFromPicHandle
 *------------------------------------------------------------------------------*
		pict	(in) PicHandle to extract the profile from
		
		index	(in) the one-based index of the profile in the
				pict to unflatten
				
		prof	(out) returns reference to profile if result is noErr
				Otherwise undefined
				
		profLoc	(in) profile location specification.  If null then a temporary
				profile is created and no disk file will remain after
				CMCloseProfile is called
\*------------------------------------------------------------------------------*/
CMError GetIndexedProfileFromPicHandle ( PicHandle pict,
										 unsigned long index,
										 CMProfileRef *prof,
										 CMProfileLocation *profLoc )
{	
	CMError				cmerr = noErr ;
	CMProfileRef		hndlProf ;
	CMProfileLocation 	hndlProfLoc ;
	Handle				h ;
	static CQDProcs		procs ;
	static GWorldPtr	offscreen ;

	cmerr = NewSmallGWorld( &offscreen ) ;
	if (cmerr) return cmerr ;
	
	SetStdCProcs( &procs ) ;
	procs.textProc    = NewQDTextProc   ( TextProc    ) ;
	procs.lineProc    = NewQDLineProc   ( LineProc    ) ;
	procs.rectProc    = NewQDRectProc   ( RectProc    ) ;
	procs.rRectProc   = NewQDRRectProc  ( RRectProc   ) ;
	procs.ovalProc    = NewQDOvalProc   ( OvalProc    ) ;
	procs.arcProc     = NewQDArcProc    ( ArcProc     ) ;
	procs.polyProc    = NewQDPolyProc   ( PolyProc    ) ;
	procs.rgnProc     = NewQDRgnProc    ( RgnProc     ) ;
	procs.bitsProc    = NewQDBitsProc   ( BitsProc    ) ;
	procs.commentProc = NewQDCommentProc( GetIndexedHandleCommentProc ) ;
	procs.txMeasProc  = NewQDTxMeasProc ( TxMeasProc  ) ;
	
	gCount = 0 ;
	gHandle = 0 ;
	gIndex = index ;
	
	cmerr = DrawPicHandleUsingBottlenecks ( pict, procs, offscreen ) ;
	
	DisposeGWorld( offscreen ) ;
	DisposeRoutineDescriptor( procs.textProc  ) ;
	DisposeRoutineDescriptor( procs.lineProc  ) ;
	DisposeRoutineDescriptor( procs.rectProc  ) ;
	DisposeRoutineDescriptor( procs.rRectProc ) ;
	DisposeRoutineDescriptor( procs.ovalProc  ) ;
	DisposeRoutineDescriptor( procs.arcProc   ) ;
	DisposeRoutineDescriptor( procs.polyProc  ) ;
	DisposeRoutineDescriptor( procs.rgnProc   ) ;
	DisposeRoutineDescriptor( procs.bitsProc  ) ;
	DisposeRoutineDescriptor( procs.commentProc ) ;
	DisposeRoutineDescriptor( procs.txMeasProc  ) ;
	
	if (cmerr) return cmerr ;
	
	h = gHandle ;
	
	if ( gHow==cmEmbedWholeProfile )
	{
		hndlProfLoc.locType = cmHandleBasedProfile ;
		hndlProfLoc.u.handleLoc.h = h ;
			
		if (!cmerr) cmerr = CMOpenProfile( &hndlProf, &hndlProfLoc ) ;
		if (!cmerr) cmerr = CMCopyProfile( prof, profLoc, hndlProf ) ;
		if (!cmerr) cmerr = CMCloseProfile( hndlProf ) ;
	}
	else if ( gHow==cmEmbedProfileIdentifier)
	{
		CMProfileIdentifierPtr	ident ;
		unsigned long			size ;
		HLock( h );
		size = GetHandleSize( h );
		ident = (CMProfileIdentifierPtr)*h ;
		cmerr = CreatePseudoProfileID( ident, &size, prof, profLoc ) ;
	}
	
	DisposeHandle( h ) ;	
	return cmerr ;
}


static pascal void GetIndexedHandleCommentProc ( short kind, short dataSize, Handle dataHandle)
{
	long		selector ;
	Ptr			dataPtr ;
	OSErr		err ;

	if (gCount > gIndex) return ;					// we allready found the profile

	switch (kind)
	{
		case cmBeginProfile:
			gCount ++ ;								// we found a version 1 profile
			if (gCount != gIndex) break ;			// not the profile we're looking for
			gHandle = dataHandle ;
			err = HandToHand( &gHandle ) ;
			gHow = cmEmbedWholeProfile ;
			break ;

		case cmComment:
			if ( dataSize <= 4 ) break;				// dataSize too small for selector so break
			selector = *((long *)(*dataHandle)) ;	// get the selector from 1st long in data
			HLock( dataHandle ) ;
			dataPtr = *dataHandle + 4 ;
			dataSize -= 4 ;
			switch (selector)
			{
				case cmBeginProfileSel :
					gCount ++ ;						// we found a version 2 profile
					if (gCount != gIndex) break ;	// not the profile we're looking for
					err = PtrToHand(dataPtr,&gHandle,dataSize);
					gHow = cmEmbedWholeProfile ;
					break ;
					
				case cmContinueProfileSel :
					if (gCount != gIndex) break ;	// not the profile we're looking for
					err = PtrAndHand(dataPtr,gHandle,dataSize);
					break ;
					
				case cmProfileIdentifierSel :
					gCount ++ ;						// we found a profileID
					if (gCount != gIndex) break ;	// not the profile we're looking for
					err = PtrToHand(dataPtr,&gHandle,dataSize);
					gHow = cmEmbedProfileIdentifier ;
					break ;

				case cmEndProfileSel :
					// check to see if we're over-reading ??
					break ;
			}
			HUnlock( dataHandle ) ;
			break ;
	}		
}


Boolean IsPseudoProfile ( CMProfileRef prof )
{
	CMError					cmerr ;
	Boolean					found ;
	
	cmerr = CMProfileElementExists( prof, cmProfIdentTag, &found) ;
	return ( found && !cmerr ) ;
}


CMError GetPseudoProfileIDHdl ( CMProfileRef prof,
								CMProfileIdentifierHdl *ident )
{
	CMError					cmerr ;
	unsigned long			identSize ;
	
	cmerr = CMGetProfileElement( prof, cmProfIdentTag,
								 &identSize, nil ) ;
	if (!cmerr)
	{
		identSize -= 8 ;
		*ident = (CMProfileIdentifierHdl)NewHandleClear( identSize );
		if (!(*ident)) cmerr = MemError();
	}
	
	HLock( (Handle)*ident );
	if (!cmerr) cmerr = CMGetPartialProfileElement( prof, cmProfIdentTag,
													8, &identSize,
													(void *)(**ident) ) ;
	HUnlock( (Handle)*ident );
	if (cmerr) DisposeHandle( (Handle)*ident );									 
	return cmerr ;
}

CMError CreatePseudoProfileID ( CMProfileIdentifierPtr ident,
								unsigned long *size,
								CMProfileRef *prof,
								CMProfileLocation *profLoc )

{
	CMError					cmerr ;
	CMDateTimeType			dtTag ;
	CMTextDescriptionType	*tdTagPtr ;
	CMProfIdentType			*idTagPtr ;

	cmerr = CMNewProfile( prof, profLoc ) ;
	
	// add the header to the psuedo profile 
	cmerr = CMSetProfileHeader( *prof,
								(CMAppleProfileHeader*)&(ident->profileHeader) ) ;
	
	// add the calibration date/time to the psuedo profile 
	dtTag.typeDescriptor = cmCalibrationDateTimeTag;
	dtTag.dateTime = ident->calibrationDate ;
	cmerr = CMSetProfileElement( *prof, cmCalibrationDateTimeTag,
								 sizeof(CMDateTimeType), (void *)(&dtTag) ) ;
								 
	// add the TextDescription to the psuedo profile 
	tdTagPtr = (CMTextDescriptionType*)NewPtrClear( 12 + ident->ASCIIProfileDescriptionLen + 14 );
	tdTagPtr->typeDescriptor = cmProfileDescriptionTag;
	tdTagPtr->ASCIICount = ident->ASCIIProfileDescriptionLen;
	BlockMoveData( ident->ASCIIProfileDescription,
				 tdTagPtr->ASCIIName,
				 tdTagPtr->ASCIICount);
	cmerr = CMSetProfileElement( *prof, cmProfileDescriptionTag,
								 GetPtrSize((Ptr)tdTagPtr), (void *)(tdTagPtr) ) ;
	DisposePtr((Ptr)tdTagPtr);
	
	// add the my custom tag to the psuedo profile 
	idTagPtr = (CMProfIdentType*)NewPtrClear( 8 + (*size) );
	idTagPtr->typeDescriptor = cmProfIdentTag;
	BlockMoveData( ident, &(idTagPtr->ident), *size );
	cmerr = CMSetProfileElement( *prof, cmProfIdentTag,
								 GetPtrSize((Ptr)idTagPtr), (void *)(idTagPtr) ) ;
	DisposePtr((Ptr)idTagPtr);
	
	return cmerr ;
}


CMError PseudoProfileFolderSearch( CMProfileRef srcProf, CMProfileRef *foundProf)
{
	CMError					cmerr ;
	unsigned long			matchedCount;
	CMProfileSearchRef		search;
	CMProfileIdentifierHdl	ident ;

	if ( CS_version() < 0x0210 )
		return unimpErr ;

	cmerr = GetPseudoProfileIDHdl( srcProf, &ident ) ;
	HLock( (Handle)ident );
	if (!cmerr) cmerr = CMProfileIdentifierFolderSearch( *ident,
													 &matchedCount, &search);
	HUnlock( (Handle)ident );
	if (ident) DisposeHandle( (Handle)ident );
	
	if (!cmerr) {
		if (matchedCount==0)
			*foundProf = nil ;
		else
			cmerr = CMSearchGetIndProfile( search, 1, foundProf) ;
	}
	
	if (search) CMDisposeProfileSearch( search );

	return cmerr ;
}


CMError PseudoProfileListSearch( CMProfileRef srcProf, CMProfileRef *foundProf,
								 CMProfileRef *profileList, unsigned long listSize )
{
	CMError					cmerr ;
	unsigned long			matchedCount;
	CMProfileIdentifierHdl	ident ;
	
	if (listSize==0)
	{
		*foundProf = nil ;
		return noErr ;
	}
	
	if ( CS_version() < 0x0210 )
		return unimpErr ;

	cmerr = GetPseudoProfileIDHdl( srcProf, &ident ) ;
	HLock( (Handle)ident );
	matchedCount=1;	// return no more than one match
	if (!cmerr) cmerr = CMProfileIdentifierListSearch( *ident,
			 									   profileList, listSize,
			  									   &matchedCount, foundProf ) ;
	HLock( (Handle)ident );
	if (ident) DisposeHandle( (Handle)ident);
	
	if (!cmerr) {
		if (matchedCount==0)
			*foundProf = nil ;
	}
	
	return cmerr ;
}


/*------------------------------------------------------------------------------*\
	ReadProfileIdentifier
 *------------------------------------------------------------------------------*
		This reads a picture file from a FSSpec into a CMProfileIdentifierHdl
\*------------------------------------------------------------------------------*/
OSErr ReadProfileIdentifier ( FSSpec spec, CMProfileIdentifierHdl *idHdl ) 
{
	long			fileSize ;
	short			ref ;
	OSErr			err ;
	
	// try to open the file
	err = FSpOpenDF( &spec, fsRdPerm, &ref ) ;
	if ( err ) return nil;

	err = GetEOF( ref, &fileSize ) ;
	if ( err ) goto bail;
	
	*idHdl = (CMProfileIdentifierHdl)NewHandle( fileSize ) ;
	if( *idHdl == nil )
	{
		err = MemError() ;
		goto bail;
	}
	
	HLock( (Handle)*idHdl ) ;
	err = FSRead( ref, &fileSize, (Ptr)**idHdl ) ;
	HUnlock( (Handle)*idHdl ) ;
	if( err )
	{
		DisposeHandle( (Handle)*idHdl ) ;
		*idHdl = nil ;
	}

bail:
	FSClose( ref ) ;
	return err ;	
}


/*------------------------------------------------------------------------------*\
	WriteProfileIdentifier
 *------------------------------------------------------------------------------*
		This writes a ProfileIdentifier file to a FSSpec from a CMProfileIdentifierHdl
		If the file referenced by the FSSpec already exists, 
		then it is deleted and a new file is created.
\*------------------------------------------------------------------------------*/
OSErr WriteProfileIdentifier ( FSSpec spec, CMProfileIdentifierHdl idHdl, OSType creator )
{
	short			refNum;
	long			inOutCount;
	OSErr			err ;

	err = FSpCreate( &spec, creator, 'pfid', 0 /*reply.sfScript*/ ) ;
	
	/* this is quick 'n' dirty or there'd be more robust handling here */
	if (err == dupFNErr)
	{
		err = FSpDelete( &spec ) ;
		err = FSpCreate( &spec, creator, 'pfid', 0 /*reply.sfScript*/ ) ;
	}
	
	err = FSpOpenDF( &spec, fsWrPerm, &refNum) ;

	if ( err == noErr )
	{ 
		inOutCount = GetHandleSize( (Handle)idHdl ) ;
		HLock( (Handle)idHdl ) ;
		err = FSWrite( refNum, &inOutCount, *idHdl ) ;
		HUnlock( (Handle)idHdl ) ;
		FSClose( refNum ) ;								/* close it */
	}
		
	return err ;
}


/**\
|**| ==============================================================================
|**| NO-OP BOTTLENECK ROUTINES
|**| ==============================================================================
\**/

static pascal void TextProc ( short byteCount, Ptr textAddr, Point numer, Point denom )
{
}
static pascal void LineProc ( Point endPoint )
{
}
static pascal void RectProc ( GrafVerb verb, Rect *r )
{
}
static pascal void RRectProc ( GrafVerb verb, Rect *r, short ovalWidth, short ovalHeight )
{
}
static pascal void OvalProc ( GrafVerb verb, Rect *r )
{
}
static pascal void ArcProc ( GrafVerb verb, Rect *r, short startAngle, short arcAngle )
{
}
static pascal void PolyProc ( GrafVerb verb, PolyHandle poly )
{
}
static pascal void RgnProc ( GrafVerb verb, RgnHandle rgn )
{
}
static pascal void BitsProc ( BitMap *bitPtr, Rect *srcRect, Rect *dstRect, short mode, RgnHandle maskRgn )
{
}
static pascal void TxMeasProc ( short byteCount, Ptr textAddr, Point *numer, Point *denom, FontInfo *info )
{
}
