// Simple framework for Macintosh sample code
//
// David Hayward 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains useful list magager routines
// 
// 11/1/95	david	first cut

#include <MacWindows.h>
#include <MacTypes.h>
#include <Drag.h>

#include "listsUtils.h"
#include "dragUtils.h"



/**\
|**| ==============================================================================
|**| PRIVATE GLOBALS
|**| ==============================================================================
\**/
static ListHandle			gCurrentClickedList ;
static StartListDragUPP		gStartListDragUPP ;
static Boolean				gDragStarted ;


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
pascal Boolean	MyClickLoop			( void ) ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------*/

Boolean LClickDrag ( EventRecord *event, ListHandle list,
					StartListDragUPP startDrag, Boolean *dragStarted )
{
	OSErr				err ;
	Rect				listRect;
	Point				localPt;
	Boolean				doubleClick;
	short				modifiers ;
	ListClickLoopUPP	oldClickLoop ;
	ListClickLoopUPP	newClickLoop ;
	
	doubleClick = false;
	*dragStarted = gDragStarted = false;
	localPt = event->where;
	GlobalToLocal(&localPt);

	if ((*list)->lActive == true)
	{
		listRect = (*list)->rView;
		if ((*list)->vScroll != NULL)	listRect.right += 15;
		if ((*list)->hScroll != NULL)	listRect.bottom += 15;
	
		if (PtInRect(localPt, &listRect) == true)
		{
			
			modifiers = (event->modifiers) & (shiftKey|cmdKey)  ;

			if ( MouseHitSelection(list,localPt)		// if click on existing selection
				  && modifiers==0 )							// and no modifiers are held down
			{
				// Within this list, we may have a cell selected that the user
				// now wants to drag.  MouseHitSelection will first check if
				// the user wants to do such a thing, and if so it will start
				// the drag with the appropriate information.
				
				// try to drop the cell(s) somewhere else.
				// if successful, clear the selection in the list.
				err = CallStartListDragProc(startDrag, list, event) ;
				if ( !err )
					ClearListSelection( list ) ;
				*dragStarted = true ;
			}
			else
			{
				// Save the current list and callback for use in our click loop 
				gCurrentClickedList = list ;
				gStartListDragUPP = startDrag ;
				
				oldClickLoop = (**list).lClickLoop ;
				newClickLoop = NewListClickLoopProc(MyClickLoop) ;
				
				(**list).lClickLoop = newClickLoop ;
				doubleClick = LClick( localPt, event->modifiers, list ) ;
				(**list).lClickLoop = oldClickLoop ;
				DisposeRoutineDescriptor( newClickLoop ) ;
			}
		}
	}
	
	return doubleClick ;
}


/*------------------------------------------------------------------*/

Boolean GetSingleSelect( Cell *cell, ListRef list)
{
	Cell	nextcell;
	
	cell->h = cell->v = 0;
	if ( LGetSelect(true, cell, list))			// if we can fine one selected item
	{
		nextcell = *cell ;
		LNextCell( false, true, &nextcell, list ) ;
		if ( !LGetSelect(true, &nextcell, list) )	// but we can't find a second item
			return true ;
	}

	return false ;
}


/*------------------------------------------------------------------*/

void GetCellRect ( ListHandle list, Cell *cell, Rect *rct)
{
	Cell	adjustedCell ;
	Rect	cellRect ;
	
	//
	// adjustedCell is needed because the current view of the list may not
	// have cell (0, 0) at the top left corner.  This takes the top and left
	// values of the visible field and subtracts them from the selected cell,
	// to give us this cell's location in the current view (honoring scroll bars).
	//

	adjustedCell.v = cell->v - (*list)->visible.top;
	adjustedCell.h = cell->h - (*list)->visible.left;

	cellRect.top = (*list)->rView.top + (adjustedCell.v * (*list)->cellSize.v);
	cellRect.bottom = cellRect.top + (*list)->cellSize.v;

	cellRect.left = (*list)->rView.left + (adjustedCell.h * (*list)->cellSize.h);
	cellRect.right = cellRect.left + (*list)->cellSize.h;

	*rct = cellRect;
}


/*------------------------------------------------------------------*/

void ClearListSelection ( ListHandle list )
{
	Boolean		notAtEnd = true ;
	Cell		cell = {0,0} ;
	
	while ( notAtEnd && LGetSelect(true,&cell,list) )
	{
		LSetSelect( false, cell, list ) ;		

		notAtEnd = LNextCell( true, true, &cell, list ) ;
	}
}


/*------------------------------------------------------------------*/

Boolean GetListSelectionRect ( ListHandle list, Rect *rct )
{
	Boolean		notAtEnd = true ;
	Boolean		emptyRct = true ;
	Cell		cell = {0,0} ;
	Rect		tempRct ;
	
	SetRect( rct, 0,0,0,0 ) ;	// make rect empty at first
	
	while ( notAtEnd && LGetSelect(true,&cell,list) )
	{
		GetCellRect( list, &cell, &tempRct) ;
		
		UnionRect( &tempRct, rct, rct ) ;
		emptyRct = false ;
		
		notAtEnd = LNextCell( true, true, &cell, list ) ;
	}
	
	return !emptyRct ;
}


/*------------------------------------------------------------------*/

Boolean GetListSelectionRgn ( ListHandle list, RgnHandle rgn )
{
	Boolean		notAtEnd = true ;
	Boolean		emptyRgn = true ;
	Cell		cell = {0,0} ;
	Rect		rect ;
	RgnHandle	tempRgn = NewRgn() ;
	
	SectRgn( rgn,tempRgn,rgn ) ;	// make rgn empty at first
	
	while ( notAtEnd && LGetSelect(true,&cell,list) )
	{
		GetCellRect( list, &cell, &rect) ;
		
		RectRgn( tempRgn, &rect) ;
		UnionRgn( tempRgn, rgn, rgn ) ;
		emptyRgn = false ;
		
		notAtEnd = LNextCell( true, true, &cell, list ) ;
	}
	
	DisposeRgn( tempRgn ) ;
	return !emptyRgn ;
}


/*------------------------------------------------------------------*/

Boolean MouseHitSelection ( ListHandle list, Point localPt )
{
	Point		globalPt;
	RgnHandle	selRgn ;
	Boolean		returnVal = false;
	
	selRgn = NewRgn();
	if (!selRgn)
		goto bail;
		
	if ( GetListSelectionRgn(list,selRgn) == false)
		goto bail;

	if ( ! PtInRgn(localPt,selRgn) )
		goto bail;

	globalPt = localPt ;
	LocalToGlobal( &globalPt );
	returnVal = WaitMouseMoved( globalPt ) ;
	
bail:
	
	if (selRgn)
		DisposeRgn( selRgn ) ;
		
	return returnVal;
}


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------*/

static pascal Boolean MyClickLoop ( void )
{
	OSErr		err ;
	EventRecord	dummyEvent;
	Point		mousePt;
	Cell		cell = {0,0} ;
	ListHandle	list;
	Rect		listRect ;

	list = gCurrentClickedList ;
	
	if ( LGetSelect(true,&cell,list) )				// if something is selected
	{
		OSEventAvail(0, &dummyEvent);
		mousePt = dummyEvent.where ;
		GlobalToLocal( &mousePt ) ;
		listRect = (**list).rView ;
		InsetRect( &listRect, -10, -10 ) ;

		if ( !PtInRect(mousePt,&listRect) )				// if mouse is outside list
		{
			long		tmpLong;
			Rect		selRect;
			
			// We go through the trouble of making sure the mouse doesn't
			// appear outside of the list cell region when the drag starts.
			// More purdy that way.
			
			GetListSelectionRect( list, &selRect ) ;
			InsetRect( &selRect, 2, 2 ) ;
			tmpLong = PinRect( &selRect, mousePt ) ;
			
			// build dummy event
			OSEventAvail(0, &dummyEvent);
			dummyEvent.what = mouseDown;
			dummyEvent.where = *(Point *) &tmpLong ;
			LocalToGlobal( &(dummyEvent.where) ) ;
			
			// try to drop the cell(s) somewhere else.
			// if successful, clear the selection in the list.
			err = CallStartListDragProc(gStartListDragUPP, list, &dummyEvent) ;
			if ( !err )
				ClearListSelection( list ) ;
			gDragStarted = true ;
			return false;						// false causes LClick to terminate
		}
	}
	return true;
}



