// ===========================================================================
//	<PP Starter Source>.cp 		1994-1995 Metrowerks Inc. All rights reserved.
// ===========================================================================
//
//	This file contains the starter code for a PowerPlant application

#include <Windows.h>
#include <CMApplication.h>

#ifndef offsetof
#define offsetof(structure,field) ((UInt32)&((structure *) 0)->field)
#endif

#if Build_as_Calibrator
#include "FreeAllMallocMemory.h"
#include "NewWithFree.h"
#endif

#include "CreateMonitorProf.h"
#include "StringUtils.h"

// declarations for resource ids (ResIDTs) here
const ResIDT	theMainWindowPaneID		= 1100;	
const ResIDT	theAlertWindowPaneID	= 1101;
	
const ResIDT	kDescriptionPaneID		= 1300;
const ResIDT	kCopyrightPaneID		= 1301;

const ResIDT	kXYZFieldPaneIDStart	= 1302;
const ResIDT	kGammaFieldPaneIDStart	= 1310;
const ResIDT	kVcgtFieldPaneIDStart	= 1313;

const short		StopAlertID				= 255;

#define CustomGetID						-4048
#define itemSFProfileName				11
#define kProfileType					'prof'

#define	msg_Save	1400
#define	msg_Load	1401
#define	msg_Cancel	1402

#define kFixed00			0x00000000
#define kFixed07			0x0000B333
#define kFixed10			0x00010000

#define FixedToDoub(x)		( ((double)(x)) / ((double)kFixed10) )
#define DoubToFixed(x)		((Fixed)((x) * 65536.0 + 0.5))
#define FixedToShFix(x)		((UInt16)(((x)+128)>>8))
#define ShFixToFixed(x)		((Fixed)(((x)<<8)+((x)&(0xF))))

#if Build_as_Calibrator
// entry functions for monitors
pascal Boolean CanCalibrate (AVIDType displayID, Str255 errMessage);
pascal OSErr Calibrate (CalibratorInfo *);
#endif



/* Types */
	
typedef struct
{
	FSSpec				lastSpec;	// last file spec that we used to display profile name
	StandardFileReply	reply;		// current reply for CustomGetFile
	OSType				dataType;	
	OSType				dataClass;	
} FilterDataStructure;
	
static pascal Boolean TheFileFilter ( CInfoPBPtr pb, FilterDataStructure *fds );
static pascal short TheDialogHook ( short item, DialogRef theDialogRef, FilterDataStructure *fds );
static void		CSProfileInfoString ( FSSpec *spec, Str255 info );
static void		SetSFDefaults ( short vRefNum, long dirID );
static void		GetSFDefaults ( short *vRefNum, long *dirID );
static OSErr	StandardGetProfile (CMProfileRef *profile, CMProfileLocation *loc, Boolean *sfGood);
static OSErr	ReadProfileCM2Header ( FSSpec *spec, CM2Header *hdr);
static Fixed	BestGammaForBytes( int count, UInt8 *s);
static Fixed	BestGammaForShorts( int count, UInt16 *s);
static char*	Pstrcpy2C (char *dst, ConstStr255Param src, const unsigned char dstLen );
static StringPtr Pstrcpy (StringPtr dst, ConstStr255Param src, const unsigned char dstLen );


/* Globals */
SInt32		gCSVersion = 0;
	

// ===========================================================================
//	 Main Program
// ===========================================================================
#if Build_as_Calibrator

//===================================================================== Functions
//--------------------------------------------------------------------- CanCalibrate

pascal Boolean CanCalibrate (AVIDType displayID, Str255 errMessage)
{
	GDHandle	theDisplay;
	OSErr		theErr;
	SInt16		depth;
	Boolean		canDo;
	
	canDo = false;
	
	// We need ColorSync 2.6 and later to run.
	if (Gestalt(gestaltColorMatchingVersion, &gCSVersion) != noErr ||
		(gCSVersion < 0x0260))
		goto bail;
	
	theErr = DMGetGDeviceByDisplayID(displayID, &theDisplay, false);
	if (theErr)
		goto bail;
	
	canDo = true;
	
	depth = (*((*theDisplay)->gdPMap))->pixelSize;
	
bail:
	
	errMessage[0] = 0;
	return canDo;
}

//--------------------------------------------------------------------- Calibrate

pascal OSErr Calibrate (CalibratorInfo* theInfo)
{
	OSErr				theErr = noErr;
	
									// Set Debugging options
	SetDebugThrow_(debugAction_Alert);
	SetDebugSignal_(debugAction_Alert);

									// Initialize standard Toolbox managers
	UQDGlobals::SetQDGlobals( (QDGlobals *) ((* (char **) SetCurrentA5 ()) -
											(sizeof (QDGlobals) - sizeof (GrafPtr)))
							);
	
	if (theInfo->profileLocationSize < sizeof(CMProfileLocation) )
		return cmInvalidProfileLocation;
	
	MakeMonitorProfApp	theApp(theInfo);// replace this with your App type
	
	theApp.Run();

				// Flag the profile as good (did the user click "Create It").
	theInfo->isGood = theApp.GetIsGood();
	if (theInfo->isGood)
		*theInfo->profileLocationPtr = theApp.GetProfileLocation();
	
	FreeAllCplusMemory();
	FreeAllMallocMemory();
	
	return theErr;
}

#else

void main ( void )
{
	CalibratorInfo theInfo;
									// Setup theInfo structure
	theInfo.displayID = 0;
	theInfo.eventProc = nil;
	theInfo.isGood = false;
	
									// Set Debugging options
	SetDebugThrow_( debugAction_Nothing );
	
	try
	{
		// Initialize Memory Manager
		InitializeHeap(3);		
				
		// Initialize standard Toolbox managers
		UQDGlobals::InitializeToolbox(&qd);
		
		// We need ColorSync 2.5 and later to run.
		if (Gestalt(gestaltColorMatchingVersion, &gCSVersion) != noErr ||
			(gCSVersion < 0x0250))
		{
			::ParamText("\pError: ColorSync is not installed.",
					"\pPlease install the ColorSync System Extension","\pand retry the application","\p");
			::StopAlert(StopAlertID,NULL);
			return;
		}
		
		// Install a GrowZone function to catch low memory situations.	
		// Parameter is the size of the memory reserve in bytes.
		//new LGrowZone(20000);			
	
		// Create the application object and run it.
		MakeMonitorProfApp theApp(&theInfo);	// replace this with your App type
		theApp.Run();
	}
	catch(...){
		SignalPStr_("\pException caught in main");
	}
}

#endif


// ---------------------------------------------------------------------------
//	 MakeMonitorProfApp 			
//	Constructor
// ---------------------------------------------------------------------------
MakeMonitorProfApp::MakeMonitorProfApp(CalibratorInfo *theInfo)
{
	displayID = theInfo->displayID;
#if Build_as_Calibrator
	eventProc = theInfo->eventProc;
	isGood = theInfo->isGood = false;
	oldMenuBar = ::GetMenuBar();
	::ClearMenuBar();
	mTarget = LCommander::sTarget;
#endif

	// Register functions classes
	RegisterClass_(LButton);
	RegisterClass_(LCaption);
	RegisterClass_(LDialogBox);
	RegisterClass_(LEditField);
	RegisterClass_(LPane);
	RegisterClass_(LStdButton);
	RegisterClass_(LView);
	RegisterClass_(LWindow);
	RegisterClass_(LTabGroup);
	RegisterClass_(LGroupBox);
	
	// Create the single application window.
	theMainWindow = MakeEventsWindow();
	ThrowIfNil_( theMainWindow );
	
	// install the double KeyFilter function.
	LEditField	*theEditField;
	int			i;
	for (i=0;i<8;i++)
	{
		theEditField = (LEditField *)theMainWindow->FindPaneByID(i+kXYZFieldPaneIDStart); 
		ThrowIfNil_(theEditField);
		theEditField->SetKeyFilter(MakeMonitorProfApp::FixedField);
	}
	for (i=0;i<3;i++)
	{
		theEditField = (LEditField *)theMainWindow->FindPaneByID(i+kGammaFieldPaneIDStart); 
		ThrowIfNil_(theEditField);
		theEditField->SetKeyFilter(MakeMonitorProfApp::FixedField);
	}
	for (i=0;i<9;i++)
	{
		theEditField = (LEditField *)theMainWindow->FindPaneByID(i+kVcgtFieldPaneIDStart); 
		ThrowIfNil_(theEditField);
		theEditField->SetKeyFilter(MakeMonitorProfApp::FixedField);
	}
	
	CMProfileRef	profile = nil;
	
	if (displayID)
		ThrowIfOSErr_( ::CMGetProfileByAVID(displayID, &profile));
	else
		ThrowIfOSErr_( ::CMGetSystemProfile(&profile));
	
	ThrowIfOSErr_( GetProfileInfo(profile));
	ThrowIfOSErr_( CloseProfile(profile,fsRdPerm));
	SetEditFields();
}


// ---------------------------------------------------------------------------
//	 ~MakeMonitorProfApp			
//	Destructor
// ---------------------------------------------------------------------------
MakeMonitorProfApp::~MakeMonitorProfApp()
{
	delete LMenuBar::GetCurrentMenuBar();
	
#if Build_as_Calibrator
	if (oldMenuBar)
		::SetMenuBar(oldMenuBar);
	::DrawMenuBar();
	
	SetUpdateCommandStatus(false);
	SwitchTarget(mTarget);
#endif
	
	delete LModelDirector::GetModelDirector();
	
	URegistrar::DisposeClassTable();
	LPeriodical::DeleteIdlerAndRepeaterQueues();
}

#if Build_as_Calibrator

// ---------------------------------------------------------------------------
//	 MakeModelDirector
// ---------------------------------------------------------------------------
//	We override this so that high level AppleEvent callbacks don't get installed

void
MakeMonitorProfApp::MakeModelDirector()
{
//	LApplication::MakeModelDirector();
}

void
MakeMonitorProfApp::EventUpdate		(const EventRecord &inMacEvent)
{
	LWindow	*theWindow = LWindow::FetchWindowObject(
									(WindowPtr) inMacEvent.message);
	if (theWindow == nil)
	{
		CalibrateEventUPP upp = GetEventProc();
		if (upp)
			CallCalibrateEventProc(upp, &inMacEvent);
	}
	else
		LApplication::EventUpdate(inMacEvent);
}

void
MakeMonitorProfApp::EventActivate		(const EventRecord &inMacEvent)
{
	LWindow	*theWindow = LWindow::FetchWindowObject(
									(WindowPtr) inMacEvent.message);
	if (theWindow == nil)
	{
		CalibrateEventUPP upp = GetEventProc();
		if (upp)
			CallCalibrateEventProc(upp, &inMacEvent);
	}
	else
		LApplication::EventActivate(inMacEvent);
}

#endif

// ---------------------------------------------------------------------------------
//	 ListenToMessage
// ---------------------------------------------------------------------------------
void
MakeMonitorProfApp::ListenToMessage( MessageT theMessage, void *ioParam )
{
#pragma unused (ioParam)
	OSErr			err = noErr;
	CMProfileRef	profile = nil;
	
	Try_ {	
		if (theMessage == msg_Load)
		{
			ThrowIfOSErr_( LoadProfile(&profile));
			ThrowIfOSErr_( GetProfileInfo(profile));
			ThrowIfOSErr_( CloseProfile(profile,fsRdPerm));
			SetEditFields();
		}
		else if (theMessage == msg_Save)
		{
			GetEditFields();
			ThrowIfOSErr_( SaveProfile(&profile));
			ThrowIfOSErr_( SetProfileInfo(profile));
			ThrowIfOSErr_( CloseProfile(profile,fsWrPerm));
			mState = programState_Quitting;
			isGood = true;
		}
		else if (theMessage == msg_Cancel)
		{
			mState = programState_Quitting;
		}
	}
	Catch_(err) {
		if (err<0) ThrowOSErr_(err);
	}
}
		
// ---------------------------------------------------------------------------
//	 AllowSubRemoval
// ---------------------------------------------------------------------------
//	We override this so that closing window is the same as a quit

Boolean
MakeMonitorProfApp::AllowSubRemoval(
							LCommander*	/* inSub */)
{
	mState = programState_Quitting;
	return true;
}


// ---------------------------------------------------------------------------------
//	 MakeEventsWindow
// ---------------------------------------------------------------------------------
LWindow *
MakeMonitorProfApp::MakeEventsWindow()
{
	// Create the window.
	LWindow		*theWindow;
	
	theWindow = LWindow::CreateWindow( theMainWindowPaneID, this );
	ThrowIfNil_( theWindow );
	
	// Link the application (the listener) with the controls in
	// the window (the broadcasters).
	UReanimator::LinkListenerToControls( this, theWindow, theMainWindowPaneID );

	// Show the window.
	theWindow->Show();

	return theWindow;
}


// ---------------------------------------------------------------------------
//	 LoadProfile			
// ---------------------------------------------------------------------------
OSErr 
MakeMonitorProfApp::LoadProfile (CMProfileRef *profile)
{
	OSErr					err = noErr;
	Boolean					sfGood;
	
	StandardGetProfile( profile, &profileLocation, &sfGood);
	if (sfGood!=1)
		err = 1;	// cancelled
	
bail:
	
	return err;
}

// ---------------------------------------------------------------------------
//	 SaveProfile			
// ---------------------------------------------------------------------------
OSErr 
MakeMonitorProfApp::SaveProfile (CMProfileRef *profile)
{
	OSErr					err = noErr;
	CMAppleProfileHeader	header;
	Boolean					deleteIt = false;
	FSSpec*					pSpec;
	
	pSpec = &profileLocation.u.fileLoc.spec;
	
	/***** get the name of the file to be saved as *****/
#if Build_as_Calibrator
	
	err = ::CMGetColorSyncFolderSpec(kOnSystemDisk, kCreateFolder, &pSpec->vRefNum, &pSpec->parID);
	if (err) goto bail;
						 
	Pstrcpy( pSpec->name, descStr, 31 );
	
	// delete the any duplicate file dialog?
	FInfo	theFInfo;
	err = FSpGetFInfo(pSpec, &theFInfo);
	
	if (err == noErr)
	{
		// delete the any duplicate file dialog?
		deleteIt = true;
	}
	else if (err == fnfErr)
		err = noErr;
	
#else
	StandardFileReply		reply;
	
	StandardPutFile("\pSave profile as:",descStr,&reply);
	if (reply.sfGood!=1)
	{
		err = 1;	// cancelled
		goto bail;
	}
	
	profileLocation.u.fileLoc.spec = reply.sfFile;
	
	// delete the old file
	if (reply.sfReplacing)
		deleteIt = true;
#endif		
	
	profileLocation.locType = cmFileBasedProfile;
	
	// delete the existing file
	if (deleteIt)
	{
		if ((err=FSpDelete(pSpec))!=noErr)
		{
			::ParamText("\pFile Error: Can't overwrite file",pSpec->name,"\p","\p");
			::StopAlert(StopAlertID,NULL);
			goto bail;
		}
	}
	
	// create the new profiles
	if ((err=::CMNewProfile(profile, &profileLocation))!=noErr)
	{
		::ParamText("\pProfile Error: Can't create new profile","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
	GetTime((DateTimeRec*)&(header.cm2.dateTime)) ; /* For copying to DateTimeRec of the profile */
	header.cm2.CMMType					= 'appl';				
	header.cm2.profileVersion			= cmCS2ProfileVersion;				
	header.cm2.profileClass				= cmDisplayClass;
	header.cm2.dataColorSpace			= cmRGBData;	
	header.cm2.profileConnectionSpace	= cmXYZData;
	header.cm2.CS2profileSignature		= cmMagicNumber;
	header.cm2.platform					= cmMacintosh;
	header.cm2.flags					= 0;
	header.cm2.deviceManufacturer		= 'appl';	
	header.cm2.deviceModel				= 0;
	header.cm2.deviceAttributes[0]		= 0;	
	header.cm2.deviceAttributes[1]		= 0;	
	header.cm2.renderingIntent			= cmPerceptualMatch;	
	header.cm2.white.X					= 0x0000F6D4;	// 0.9642	
	header.cm2.white.Y					= 0x00010000;	// 1.0000
	header.cm2.white.Z					= 0x0000D32B;	// 0.8249
	
	if ((err=::CMSetProfileHeader(*profile, &header))!=noErr)
	{
		::ParamText("\pProfile Error: Can't create new profile","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
bail:
	
	return err;
}

// ---------------------------------------------------------------------------
//	 CloseProfile			
// ---------------------------------------------------------------------------
OSErr
MakeMonitorProfApp::CloseProfile(CMProfileRef profile, long permission)
{
	OSErr	err = noErr;
	
	if (permission==fsWrPerm && (err=::CMUpdateProfile(profile))!=noErr)
	{
		::ParamText("\pProfile Error: Can't CMUpdateProfile","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	if ((err=::CMCloseProfile(profile))!=noErr)
	{
		::ParamText("\pProfile Error: Can't CMCloseProfile","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
bail:
	
	return err;
}



// ---------------------------------------------------------------------------
//	 GetProfileInfo			
// ---------------------------------------------------------------------------
OSErr 
MakeMonitorProfApp::GetProfileInfo(CMProfileRef profile)
{
	OSErr					err = noErr;
	CMFixedXYZColor			rColor,gColor,bColor,wColor;
	CMCurveType				*rTRC=nil,*gTRC=nil,*bTRC=nil;
	CMVideoCardGamma		*vcgt=nil;
	double					rX,rY,rZ,gX,gY,gZ,bX,bY,bZ;
	double					rx,ry, gx,gy, bx,by, wx,wy;
	unsigned long			tagSz;
	ScriptCode				code;
	unsigned long			tmpVal;
	
	// read the XYZ values
	if ((err=GetProfileElementXYZ(profile,cmRedColorantTag,&rColor))!=noErr)
		goto bail;
	if ((err=GetProfileElementXYZ(profile,cmGreenColorantTag,&gColor))!=noErr)
		goto bail;
	if ((err=GetProfileElementXYZ(profile,cmBlueColorantTag,&bColor))!=noErr)
		goto bail;
	if ((err=GetProfileElementXYZ(profile,cmMediaWhitePointTag,&wColor))!=noErr)
		goto bail;
	
	// convert to tristimulus values
	
	tmpVal = (rColor.X + gColor.X + bColor.X);
	if (tmpVal==0) return paramErr;
	rX = (double)rColor.X * (double)wColor.X / (double)tmpVal;
	gX = (double)gColor.X * (double)wColor.X / (double)tmpVal;
	bX = (double)bColor.X * (double)wColor.X / (double)tmpVal;
	
	tmpVal = (rColor.Y+gColor.Y+bColor.Y);
	if (tmpVal==0) return paramErr;
	rY = (double)rColor.Y * (double)wColor.Y / (double)tmpVal;
	gY = (double)gColor.Y * (double)wColor.Y / (double)tmpVal;
	bY = (double)bColor.Y * (double)wColor.Y / (double)tmpVal;
	
	tmpVal = (rColor.Z+gColor.Z+bColor.Z);
	if (tmpVal==0) return paramErr;
	rZ = (double)rColor.Z * (double)wColor.Z / (double)tmpVal;
	gZ = (double)gColor.Z * (double)wColor.Z / (double)tmpVal;
	bZ = (double)bColor.Z * (double)wColor.Z / (double)tmpVal;
	
	tmpVal = (rX+rY+rZ);
	if (tmpVal==0) return paramErr;
	rx = (rX)/(double)tmpVal;
	ry = (rY)/(double)tmpVal;
	
	tmpVal = (gX+gY+gZ);
	if (tmpVal==0) return paramErr;
	gx = (gX)/(double)tmpVal;
	gy = (gY)/(double)tmpVal;
	
	tmpVal = (bX+bY+bZ);
	if (tmpVal==0) return paramErr;
	bx = (bX)/(double)tmpVal;
	by = (bY)/(double)tmpVal;
	
	// now since profile always have d50 relative, we use the whitepoint
	// tag to calculate white instead of Xr+Xg+Xb from above
	tmpVal = (wColor.X + wColor.Y + wColor.Z);
	if (tmpVal==0) return paramErr;
	wx = (double)(wColor.X)/(double)tmpVal;
	wy = (double)(wColor.Y)/(double)tmpVal;
	
	// 
	xyzVals[0] = DoubToFixed(rx);
	xyzVals[1] = DoubToFixed(ry);
	xyzVals[2] = DoubToFixed(gx);
	xyzVals[3] = DoubToFixed(gy);
	xyzVals[4] = DoubToFixed(bx);
	xyzVals[5] = DoubToFixed(by);
	xyzVals[6] = DoubToFixed(wx);
	xyzVals[7] = DoubToFixed(wy);
	
	/***** read the gamma values *****/
	if ((err=GetProfileElementTRC(profile,cmRedTRCTag,&rTRC))!=noErr)
		goto bail;
	
	if ((err=GetProfileElementTRC(profile,cmGreenTRCTag,&gTRC))!=noErr)
		goto bail;
	
	if ((err=GetProfileElementTRC(profile,cmBlueTRCTag,&bTRC))!=noErr)
		goto bail;
	
	/***** convert to gamma values *****/
	ExtractGammaFromTRC(rTRC,&(gammaVals[0]));
	ExtractGammaFromTRC(gTRC,&(gammaVals[1]));
	ExtractGammaFromTRC(bTRC,&(gammaVals[2]));
	
	/***** read the video card gamma *****/
	(void) GetProfileElementVCGT(profile,cmVideoCardGammaTag,&vcgt);
	
	/***** convert to gamma values *****/
	(void) ExtractGammaFromVCGT(vcgt);
	
	/***** get the profile description *****/
	if ((err=::CMGetScriptProfileDescription(profile, descStr, &code))!=noErr)
	{
		::ParamText("\pProfile Error: Can't get CMGetScriptProfileDescription","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
	/***** get the copyright tag *****/
	if ((err=::CMGetProfileElement(profile,cmCopyrightTag,&tagSz,(void *)NULL))!=noErr)
	{
		::ParamText("\pProfile Error: Can't get cmCopyrightTag","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	unsigned long tmpSz;
	copyrgtStr[0] = ((tagSz-8)>255 ? (255):(tagSz-8));
	tmpSz = copyrgtStr[0];
	if ((err=::CMGetPartialProfileElement(profile, cmCopyrightTag, 8, &tmpSz, copyrgtStr+1))!=noErr)
	{
		::ParamText("\pProfile Error: Can't get cmCopyrightTag","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
bail:
	
	if (rTRC) DisposePtr((Ptr)rTRC);
	if (gTRC) DisposePtr((Ptr)gTRC);
	if (bTRC) DisposePtr((Ptr)bTRC);
	if (vcgt) DisposePtr((Ptr)vcgt);
	return err;
}

// ---------------------------------------------------------------------------
//	 SetProfileInfo			
// ---------------------------------------------------------------------------
OSErr 
MakeMonitorProfApp::SetProfileInfo(CMProfileRef profile)
{
	OSErr					err = noErr;
	CMFixedXYZColor			theColor;
	double			z;
	double			XL,YL,ZL, XM,YM,ZM, XN,YN,ZN, XW,YW,ZW;
	double			rx,ry, gx,gy, bx,by, wx,wy;
	CMCurveType				theCurve;
	CMVideoCardGammaType	theVcgt;
	unsigned long			tagSz;
	
	// Setup
	rx = FixedToDoub(xyzVals[0]);
	ry = FixedToDoub(xyzVals[1]);
	gx = FixedToDoub(xyzVals[2]);
	gy = FixedToDoub(xyzVals[3]);
	bx = FixedToDoub(xyzVals[4]);
	by = FixedToDoub(xyzVals[5]);
	wx = FixedToDoub(xyzVals[6]);
	wy = FixedToDoub(xyzVals[7]);
	
	// Check params
	if(ry == 0) return paramErr;
	if(gy == 0) return paramErr;
	if(by == 0) return paramErr;
	if(wy == 0) return paramErr;
	
	// Calculation
	z  = wy*( (gx-bx)*ry-(rx-bx)*gy+(rx-gx)*by );
	if(z == 0) return paramErr;
	
	YL = ry*( (gx-bx)*wy-(wx-bx)*gy+(wx-gx)*by )/z;
	YM =-gy*( (rx-bx)*wy-(wx-bx)*ry+(wx-rx)*by )/z;
	YN = by*( (rx-gx)*wy-(wx-gx)*ry+(wx-rx)*gy )/z;
	
	XL = YL*rx/ry;
	ZL = YL*((1-rx)/ry - 1);
	
	XM = YM*gx/gy;
	ZM = YM*((1-gx)/gy - 1);
	
	XN = YN*bx/by;
	ZN = YN*((1-bx)/by - 1);
	
#if 0
	XW = XL+XM+XN;
	YW = 1.0; // YL+YM+YN == 1.0
	ZW = ZL+ZM+ZN;
#else
	XW = wx/wy;
	YW = 1.0;
	ZW = (1.0-wx-wy)/wy;
#endif
	
	// Scale profile from {XW,YW,ZW}-relative to d50-relative
	if (XW==0 || ZW==0) return paramErr;
	XL *= 0.9642/XW;	XM *= 0.9642/XW;	XN *= 0.9642/XW;
	ZL *= 0.8249/ZW;	ZM *= 0.8249/ZW;	ZN *= 0.8249/ZW;
	
	/***** put it into the profile *****/
	theColor.X = DoubToFixed(XL);
	theColor.Y = DoubToFixed(YL);
	theColor.Z = DoubToFixed(ZL);
	if ((err=SetProfileElementXYZ(profile,cmRedColorantTag,&theColor))!=noErr)
		goto bail;
	
	theColor.X = DoubToFixed(XM);
	theColor.Y = DoubToFixed(YM);
	theColor.Z = DoubToFixed(ZM);
	if ((err=SetProfileElementXYZ(profile,cmGreenColorantTag,&theColor))!=noErr)
		goto bail;
	
	theColor.X = DoubToFixed(XN);
	theColor.Y = DoubToFixed(YN);
	theColor.Z = DoubToFixed(ZN);
	if ((err=SetProfileElementXYZ(profile,cmBlueColorantTag,&theColor))!=noErr)
		goto bail;
	
	theColor.X = DoubToFixed(XW);
	theColor.Y = DoubToFixed(YW);
	theColor.Z = DoubToFixed(ZW);
	if ((err=SetProfileElementXYZ(profile,cmMediaWhitePointTag,&theColor))!=noErr)
		goto bail;
	
	/***** put gamma into the profile *****/
	tagSz = sizeof(CMCurveType);
	theCurve.typeDescriptor = cmSigCurveType;
	theCurve.reserved = 0x00000000;
	theCurve.countValue = 1;
	theCurve.data[0] = FixedToShFix(gammaVals[0]);
	if ((err=::CMSetProfileElement(profile,cmRedTRCTag,tagSz,(void *)&theCurve))!=noErr)
	{
		::ParamText("\pProfile Error: Can't set cmRedTRCTag","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	tagSz = sizeof(CMCurveType);
	theCurve.typeDescriptor = cmSigCurveType;
	theCurve.reserved = 0x00000000;
	theCurve.countValue = 1;
	theCurve.data[0] = FixedToShFix(gammaVals[1]);
	if ((err=::CMSetProfileElement(profile,cmGreenTRCTag,tagSz,(void *)&theCurve))!=noErr)
	{
		::ParamText("\pProfile Error: Can't set cmGreenTRCTag","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	tagSz = sizeof(CMCurveType);
	theCurve.typeDescriptor = cmSigCurveType;
	theCurve.reserved = 0x00000000;
	theCurve.countValue = 1;
	theCurve.data[0] = FixedToShFix(gammaVals[2]);
	if ((err=::CMSetProfileElement(profile,cmBlueTRCTag,tagSz,(void *)&theCurve))!=noErr)
	{
		::ParamText("\pProfile Error: Can't set cmBlueTRCTag","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
	/***** put video gamma into the profile *****/
	tagSz = sizeof(CMVideoCardGammaType);
	theVcgt.typeDescriptor = cmSigVideoCardGammaType;
	theVcgt.reserved = 0x00000000;
	theVcgt.gamma.tagType = cmVideoCardGammaFormulaType;
	theVcgt.gamma.u.formula.redGamma	= vcgtVals[0];
	theVcgt.gamma.u.formula.redMin		= vcgtVals[1];
	theVcgt.gamma.u.formula.redMax		= vcgtVals[2];
	theVcgt.gamma.u.formula.greenGamma	= vcgtVals[3];
	theVcgt.gamma.u.formula.greenMin	= vcgtVals[4];
	theVcgt.gamma.u.formula.greenMax	= vcgtVals[5];
	theVcgt.gamma.u.formula.blueGamma	= vcgtVals[6];
	theVcgt.gamma.u.formula.blueMin		= vcgtVals[7];
	theVcgt.gamma.u.formula.blueMax		= vcgtVals[8];
	if ((err=::CMSetProfileElement(profile,cmVideoCardGammaTag,tagSz,(void *)&theVcgt))!=noErr)
	{
		::ParamText("\pProfile Error: Can't set cmVideoCardGammaTag","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
	/***** put description into the profile *****/
	if ((err=SetProfileDescription(profile, descStr, 0))!=noErr)
	{
		::ParamText("\pProfile Error: Can't set profile description","\p","\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	/***** set the copyright tag *****/
	if ((err=SetProfileElementText(profile, cmCopyrightTag, copyrgtStr))!=noErr)
	{
			::ParamText("\pProfile Error: Can't set profile copyright","\p","\p","\p");
			::StopAlert(StopAlertID,NULL);
			goto bail;
	}
	
bail:
	
	return err;
}

void 
MakeMonitorProfApp::GetEditFields()
{
	LEditField		*theEditField;
	Str255			aStr;
	int				i;
	
	for (i=0;i<8;i++)
	{
		theEditField = (LEditField *)theMainWindow->FindPaneByID(i+kXYZFieldPaneIDStart); 
		ThrowIfNil_(theEditField);
		theEditField->GetDescriptor(aStr);
		P2Fixed(aStr,&(xyzVals[i]));
	}
	for (i=0;i<3;i++)
	{
		theEditField = (LEditField *)theMainWindow->FindPaneByID(i+kGammaFieldPaneIDStart); 
		ThrowIfNil_(theEditField);
		theEditField->GetDescriptor(aStr);
		P2Fixed(aStr,&(gammaVals[i]));
	}
	for (i=0;i<9;i++)
	{
		theEditField = (LEditField *)theMainWindow->FindPaneByID(i+kVcgtFieldPaneIDStart); 
		ThrowIfNil_(theEditField);
		theEditField->GetDescriptor(aStr);
		P2Fixed(aStr,&(vcgtVals[i]));
	}
	theEditField = (LEditField *)theMainWindow->FindPaneByID(kDescriptionPaneID); 
	ThrowIfNil_(theEditField);
	theEditField->GetDescriptor(descStr);

	theEditField = (LEditField *)theMainWindow->FindPaneByID(kDescriptionPaneID+1); 
	ThrowIfNil_(theEditField);
	theEditField->GetDescriptor(copyrgtStr);
}

void 
MakeMonitorProfApp::SetEditFields()
{
	LEditField		*theEditField;
	Str255			aStr;
	int				i;
	
	for (i=0;i<8;i++)
	{
		theEditField = (LEditField *)theMainWindow->FindPaneByID(i+kXYZFieldPaneIDStart); 
		ThrowIfNil_(theEditField);
		Fixed2P(xyzVals[i],aStr);
		theEditField->SetDescriptor(aStr);
	}
	for (i=0;i<3;i++)
	{
		theEditField = (LEditField *)theMainWindow->FindPaneByID(i+kGammaFieldPaneIDStart); 
		ThrowIfNil_(theEditField);
		Fixed2P(gammaVals[i],aStr);
		theEditField->SetDescriptor(aStr);
	}
	for (i=0;i<9;i++)
	{
		theEditField = (LEditField *)theMainWindow->FindPaneByID(i+kVcgtFieldPaneIDStart); 
		ThrowIfNil_(theEditField);
		Fixed2P(vcgtVals[i],aStr);
		theEditField->SetDescriptor(aStr);
	}
	theEditField = (LEditField *)theMainWindow->FindPaneByID(kDescriptionPaneID); 
	ThrowIfNil_(theEditField);
	theEditField->SetDescriptor(descStr);
	theEditField = (LEditField *)theMainWindow->FindPaneByID(kCopyrightPaneID); 
	ThrowIfNil_(theEditField);
	theEditField->SetDescriptor(copyrgtStr);
}

// ---------------------------------------------------------------------------
//	 ExtractBestGammaForTRC			
// ---------------------------------------------------------------------------
OSErr 
MakeMonitorProfApp::ExtractGammaFromTRC(CMCurveType *theTbl,Fixed *theGamma)
{
	UInt32		countValue;
	
	countValue = theTbl->countValue;
	
	if (countValue==0)
		*theGamma = kFixed10;
	else if (countValue==1)
		*theGamma = ShFixToFixed(theTbl->data[0]);
	else if (countValue==2)
		*theGamma = kFixed10;
	else // (countValue >= 3)
		*theGamma = BestGammaForShorts( countValue, theTbl->data);
	
	return noErr;
}



// ---------------------------------------------------------------------------
//	 ExtractGammaFromVCGT			
// ---------------------------------------------------------------------------
OSErr 
MakeMonitorProfApp::ExtractGammaFromVCGT(CMVideoCardGamma *vcgt)
{
	OSErr	theErr=noErr;
	
	vcgtVals[0] = vcgtVals[3] = vcgtVals[6] = kFixed07;
	vcgtVals[1] = vcgtVals[4] = vcgtVals[7] = kFixed00;
	vcgtVals[2] = vcgtVals[5] = vcgtVals[8] = kFixed10;
		
	if (vcgt==nil)
		theErr = paramErr;
	if (theErr) goto bail;
	
	if (vcgt->tagType == cmVideoCardGammaFormulaType)
	{
		vcgtVals[0] = vcgt->u.formula.redGamma;
		vcgtVals[1] = vcgt->u.formula.redMin;
		vcgtVals[2] = vcgt->u.formula.redMax;
		vcgtVals[3] = vcgt->u.formula.greenGamma;
		vcgtVals[4] = vcgt->u.formula.greenMin;
		vcgtVals[5] = vcgt->u.formula.greenMax;
		vcgtVals[6] = vcgt->u.formula.blueGamma;
		vcgtVals[7] = vcgt->u.formula.blueMin;
		vcgtVals[8] = vcgt->u.formula.blueMax;
	}
	else if (vcgt->tagType == cmVideoCardGammaTableType)
	{
		UInt32		entryCount;
		UInt32		channels;
		UInt32		entrySize;
		
		entryCount	= vcgt->u.table.entryCount;
		channels	= vcgt->u.table.channels;
		entrySize	= vcgt->u.table.entrySize;
		
		if (entryCount<2 || (entrySize!=1 && entrySize!=2) || (channels!=1 && channels!=3) )
			theErr = paramErr;
		
				// Calculate maxs and mins
		if (entrySize==1 && channels==1)
		{
			vcgtVals[1] = vcgtVals[4] = vcgtVals[7] = DoubToFixed(((double)*(UInt8*)(vcgt->u.table.data+0))/255.0);
			vcgtVals[2] = vcgtVals[5] = vcgtVals[8] = DoubToFixed(((double)*(UInt8*)(vcgt->u.table.data+(entryCount-1)))/255.0);
		}
		if (entrySize==1 && channels==3)
		{
			vcgtVals[1] = DoubToFixed(((double)*(UInt8*)(vcgt->u.table.data+(0*entryCount)))/255.0);
			vcgtVals[2] = DoubToFixed(((double)*(UInt8*)(vcgt->u.table.data+(1*entryCount-1)))/255.0);
			vcgtVals[4] = DoubToFixed(((double)*(UInt8*)(vcgt->u.table.data+(1*entryCount)))/255.0);
			vcgtVals[5] = DoubToFixed(((double)*(UInt8*)(vcgt->u.table.data+(2*entryCount-1)))/255.0);
			vcgtVals[7] = DoubToFixed(((double)*(UInt8*)(vcgt->u.table.data+(2*entryCount)))/255.0);
			vcgtVals[8] = DoubToFixed(((double)*(UInt8*)(vcgt->u.table.data+(3*entryCount-1)))/255.0);
		}
		else if (entrySize==2 && channels==1)
		{
			vcgtVals[1] = vcgtVals[4] = vcgtVals[7] = DoubToFixed(((double)*(UInt16*)(vcgt->u.table.data+0))/65535.0);
			vcgtVals[2] = vcgtVals[5] = vcgtVals[8] = DoubToFixed(((double)*(UInt16*)(vcgt->u.table.data+(entryCount-1)))/65535.0);
		}
		else if (entrySize==2 && channels==3)
		{
			vcgtVals[1] = DoubToFixed(((double)*(UInt16*)(vcgt->u.table.data+(0*entryCount)))/65535.0);
			vcgtVals[2] = DoubToFixed(((double)*(UInt16*)(vcgt->u.table.data+(1*entryCount-1)))/65535.0);
			vcgtVals[4] = DoubToFixed(((double)*(UInt16*)(vcgt->u.table.data+(1*entryCount)))/65535.0);
			vcgtVals[5] = DoubToFixed(((double)*(UInt16*)(vcgt->u.table.data+(2*entryCount-1)))/65535.0);
			vcgtVals[7] = DoubToFixed(((double)*(UInt16*)(vcgt->u.table.data+(2*entryCount)))/65535.0);
			vcgtVals[8] = DoubToFixed(((double)*(UInt16*)(vcgt->u.table.data+(3*entryCount-1)))/65535.0);
		}
		
				// Calculate gammas
		if (entryCount==2)
		{
			vcgtVals[0] = vcgtVals[3] = vcgtVals[6] = kFixed10;
		}
		else
		{
			if (entrySize==1 && channels==1)
			{
				vcgtVals[0] = vcgtVals[3] = vcgtVals[6] = BestGammaForBytes(entryCount, (UInt8*)(vcgt->u.table.data+0));
			}
			if (entrySize==1 && channels==3)
			{
				vcgtVals[0] = BestGammaForBytes(entryCount, (UInt8*)(vcgt->u.table.data+(0*entryCount)));
				vcgtVals[3] = BestGammaForBytes(entryCount, (UInt8*)(vcgt->u.table.data+(1*entryCount)));
				vcgtVals[6] = BestGammaForBytes(entryCount, (UInt8*)(vcgt->u.table.data+(2*entryCount)));
			}
			else if (entrySize==2 && channels==1)
			{
				vcgtVals[0] = vcgtVals[3] = vcgtVals[6] = BestGammaForShorts(entryCount, (UInt16*)(vcgt->u.table.data+0));
			}
			else if (entrySize==2 && channels==3)
			{
				vcgtVals[0] = BestGammaForShorts(entryCount, (UInt16*)(vcgt->u.table.data+(0*entryCount)));
				vcgtVals[3] = BestGammaForShorts(entryCount, (UInt16*)(vcgt->u.table.data+(1*entryCount)));
				vcgtVals[6] = BestGammaForShorts(entryCount, (UInt16*)(vcgt->u.table.data+(2*entryCount)));
			}
		}
	}
	else
		theErr = paramErr;
	
bail:
	
	return theErr;
}

						
// ---------------------------------------------------------------------------
//	 SetProfileDescription			
// ---------------------------------------------------------------------------
CMError
MakeMonitorProfApp::SetProfileDescription (CMProfileRef prof, Str255 mName, ScriptCode mCode)
{
	char	aName[256] = "Default name";
	UInt32	aCount = 13;
	
	if (mCode==0)
	{
		Pstrcpy2C (aName, mName, 255);
		aCount = mName[0] + 1;
	}
	
	if (gCSVersion >= 0x0260)
	{
		return CMSetProfileDescriptions(prof, aName,aCount, mName,mCode, nil,0);
	}
	else
	{
		CMError		err;
		Ptr			tag;
		UInt32		offset;
		UInt32		mCount;
		
	//	4	OSType 			typeDescriptor;		/* cmSigProfileDescriptionType */
	//	4	UInt32 			reserved;			/* fill with 0x00 */
	//	4	UInt32 			ASCIICount;			/* count of 1 byte characters */
	//	n+1	unsigned char 	ASCIIName[2];		/* Variable size*/
	//	4	UInt32 			UniCodeCode;
	//	4	UInt32 			UniCodeCount;		/* count of 2 byte characters */
	//	0	unsigned char 	UniCodeName[2];		/* Variable size */
	//	2	short 			ScriptCodeCode;
	//	1	unsigned char 	ScriptCodeCount;	/* count of 1 byte characters */
	//	n+1	unsigned char 	ScriptCodeName[2];	/* Variable size */

		offset = 0;
		mCount = mName[0]+1;
		tag = NewPtrClear( 23 + aCount + mCount );
		if (!tag) return memFullErr;
		
		// OSType  tag.typeDescriptor
		*((OSType*)(tag+offset)) = cmSigProfileDescriptionType;
		offset += 4;
		
		// UInt32  tag.reserved
		offset += 4;

		// UInt32  tag.ASCIICount
		*((UInt32*)(tag+offset)) = aCount;
		offset += 4;
		
		// tag.ASCIIName
		Pstrcpy2C(tag+offset, mName, 255 );
		offset += aCount;
		
		// tag.UniCodeCode, UniCodeCount, UniCodeName
		offset += 8;
		
		// tag.ScriptCodeCode
		*((SInt16*)(tag+offset)) = mCode;
		offset += 2;
		
		// tag.ScriptCodeCount
		*((UInt8*)(tag+offset)) = mCount;
		offset += 1;
		
		// tag.ScriptCodeName
		Pstrcpy2C(tag+offset, mName, 255 );
		
		err = CMSetProfileElement(prof, cmSigProfileDescriptionType, GetPtrSize(tag), tag);
		
		DisposePtr(tag);
		return err;
	}
}

/******************************************************************************/
CMError
MakeMonitorProfApp::SetProfileElementText ( CMProfileRef prof, OSType tagType, Str255 text )
{
	CMError		err;
	CMTextType	tag;
	
	tag.typeDescriptor = cmSigTextType;
	tag.reserved = 0L;
	
	err = CMSetPartialProfileElement(prof, tagType, 0, offsetof(CMTextType,text), &tag);
	if (!err)
		err = CMSetPartialProfileElement(prof, tagType, offsetof(CMTextType,text), text[0], text+1);
	
	return err;
}


/******************************************************************************/
CMError
MakeMonitorProfApp::GetProfileElementXYZ ( CMProfileRef prof, OSType tagType, CMFixedXYZColor *xyz )
{
	CMError			err;
	UInt32			tagSz = sizeof(CMFixedXYZColor);
	unsigned char	tagStr[] = "\p'xxxx'";
	
	err = CMGetPartialProfileElement(prof, tagType, 8, &tagSz, (void *)xyz);
	
	if (err)
	{
		(*((OSType*)(tagStr+2))) = tagType;
		::ParamText("\pProfile Error: Can't get ",tagStr,"\p collorant tag.","\p");
		::StopAlert(StopAlertID,NULL);
	}
	
	return err;
}

/******************************************************************************/
CMError
MakeMonitorProfApp::SetProfileElementXYZ ( CMProfileRef prof, OSType tagType, CMFixedXYZColor *xyz )
{
	CMError			err;
	CMXYZType		tag;
	unsigned char	tagStr[] = "\p'xxxx'";
	
	tag.typeDescriptor = cmSigXYZType;
	tag.reserved = 0L;
	tag.XYZ[0] = *xyz;
	
	err = CMSetProfileElement(prof, tagType, sizeof(tag), &tag);
	
	if (err)
	{
		(*((OSType*)(tagStr+2))) = tagType;
		::ParamText("\pProfile Error: Can't set ",tagStr,"\p collorant tag.","\p");
		::StopAlert(StopAlertID,NULL);
	}
	
	return err;
}

CMError
MakeMonitorProfApp::GetProfileElementTRC ( CMProfileRef prof, OSType tagType, CMCurveType **trc )
{
	CMError			err;
	UInt32			tagSz;
	unsigned char	tagStr[] = "\p'xxxx'";
	
	if ((err=::CMGetProfileElement(prof,tagType,&tagSz,nil))!=noErr)
	{
		(*((OSType*)(tagStr+2))) = tagType;
		::ParamText("\pProfile Error: Can't get size of ",tagStr,"\p curve tag.","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
	if ((*trc=(CMCurveType *)NewPtr(tagSz))==NULL)
	{
		Str15	errStr;
		::NumToString(err=MemError(),errStr);
		::ParamText("\pMemory Error: ",errStr,"\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
	if ((err=::CMGetProfileElement(prof,tagType,&tagSz,(void *)*trc))!=noErr)
	{
		(*((OSType*)(tagStr+2))) = tagType;
		::ParamText("\pProfile Error: Can't get ",tagStr,"\p curve tag.","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
bail:

	return err;
}

CMError
MakeMonitorProfApp::GetProfileElementVCGT ( CMProfileRef prof, OSType tagType, CMVideoCardGamma **vcgt )
{
	CMError			err;
	UInt32			tagSz;
	unsigned char	tagStr[] = "\p'xxxx'";
	
	
	if ((err=::CMGetProfileElement(prof,tagType,&tagSz,nil))!=noErr)
	{
		(*((OSType*)(tagStr+2))) = tagType;
		::ParamText("\pProfile Error: Can't get size of ",tagStr,"\p tag.","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
	tagSz-=8;
	
	if ((*vcgt=(CMVideoCardGamma *)NewPtr(tagSz))==NULL)
	{
		Str15	errStr;
		::NumToString(err=MemError(),errStr);
		::ParamText("\pMemory Error: ",errStr,"\p","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
	if ((err=::CMGetPartialProfileElement(prof, tagType, 8, &tagSz, (void *)*vcgt))!=noErr)
	{
		(*((OSType*)(tagStr+2))) = tagType;
		::ParamText("\pProfile Error: Can't get ",tagStr,"\p tag.","\p");
		::StopAlert(StopAlertID,NULL);
		goto bail;
	}
	
bail:

	return err;
}

// ---------------------------------------------------------------------------
//	 P2Fixed (assume the string only has numerical and '.' characters)	
// ---------------------------------------------------------------------------
void 
MakeMonitorProfApp::P2Fixed(StringPtr str,Fixed* f)
{
	(void) UnformatFixed(str, 128, f);
}

// ---------------------------------------------------------------------------
//	 Fixed2P			
// ---------------------------------------------------------------------------
void 
MakeMonitorProfApp::Fixed2P(Fixed f,StringPtr str)
{
	(void) FormatFixed(f, 128, str);
}


// ---------------------------------------------------------------------------
//	 FixedField
// ---------------------------------------------------------------------------
//	Key Filter for double characters
//
//		Just like NegativeIntegerField but also allow decimal char

EKeyStatus
MakeMonitorProfApp::FixedField(
	TEHandle		inMacTEH,
	Char16			inKeyCode,
	Char16			&ioCharCode,
	EventModifiers	inModifiers)
{
	Char16				decChar;
	NumberParts			parts;
	Intl0Hndl			itl4;
	SInt32				offset, length;

	decChar = '.';

	// Get default Number Parts resource/offset/length.
	GetIntlResourceTable(iuCurrentScript, smNumberPartsTable, (Handle*)&itl4, &offset, &length);
	if (itl4)
	{
		// Retrieve default Number Parts Table.
		parts = *(NumberPartsPtr)(*((Handle)itl4) + offset);
		
		// Get the decimal char from the parts
		decChar = parts.data[tokDecPoint-1].b;
	}

	if (ioCharCode == decChar) {
		return keyStatus_Input;
	} else {
		return IntegerField(inMacTEH, inKeyCode, ioCharCode, inModifiers);
	}
}


static OSErr StandardGetProfile (CMProfileRef *profile, CMProfileLocation *loc, Boolean *sfGood)
{
	Point				position;
	FilterDataStructure	fds;
	OSType				profileType = kProfileType;
	CMError				err=noErr;
	FileFilterYDUPP		fileUPP;
	DlgHookYDUPP		hookUPP;
	short				oldVRefNum;
	long				oldDirID;
	
	*sfGood = false;
	
	// Set Standard File directory to be ColorSync profiles folder
	GetSFDefaults( &oldVRefNum, &oldDirID );

	fds.lastSpec.name[0] = 0;
	fds.dataType = cmRGBData;	
	fds.dataClass = cmDisplayClass;	
	position.h = position.v = -1;
	
	fileUPP = NewFileFilterYDProc(TheFileFilter);
	hookUPP = NewDlgHookYDProc(TheDialogHook);

	// Do standard file, with our filters causing only valid system profiles to be displayed.

	CustomGetFile(	fileUPP,		// FileFilterYDUPP fileFilter
					1,				// numTypes
					&profileType,	// typeList
					&fds.reply,		// reply
					CustomGetID,	// short dlgID
					position,		// Point where
					hookUPP,		// DlgHookYDUPP dlgHook
					nil,			// ModalFilterYDUPP filterProc
					0,				// ActivationOrderListPtr activeList
					nil,			// ActivateYDUPP activate
					&fds );			// void *yourDataPtr
	
	DisposeRoutineDescriptor(fileUPP);
	DisposeRoutineDescriptor(hookUPP);

	// Restore Standard File default directory.
	SetSFDefaults(oldVRefNum,oldDirID);
	
	if (fds.reply.sfGood)
	{
		*sfGood = true;
		loc->locType = cmFileBasedProfile;
		loc->u.fileLoc.spec = fds.reply.sfFile;
		err = CMOpenProfile( profile, loc );
	}
	
	return err;
}


/* _________________________________________________________________________

	Our file filter for allowing only valid system profiles to be displayed
	in the Standard File dialog.
 _________________________________________________________________________ */

static pascal Boolean TheFileFilter ( CInfoPBPtr pb, FilterDataStructure *fds )
{
	Boolean			goodFile;
	OSErr			err;
	FSSpec			spec,fromSpec;
	CM2Header		hdr2;

	goodFile = false;
	err = noErr;
	
	if ( (*pb).hFileInfo.ioFlFndrInfo.fdType != kProfileType)
		err=1;  // this should never happen
	
	if (!err)
		err = FSMakeFSSpec( (*pb).hFileInfo.ioVRefNum,
							(*pb).dirInfo.ioDrParID,
							(*pb).hFileInfo.ioNamePtr, &fromSpec);
	
	if (!err)
//		err = ResolveFSSpec( &fromSpec, &spec );
		spec = fromSpec;
	
	if (!err)
		err = ReadProfileCM2Header( &spec, &hdr2 );

	if (!err)
		if ((hdr2.CS2profileSignature == cmMagicNumber) &&
			((hdr2.profileVersion & 0xFFFF0000) == cmCS2ProfileVersion) &&
			(fds->dataType==0 || fds->dataType==hdr2.dataColorSpace) &&
			(fds->dataClass==0 || fds->dataClass==hdr2.profileClass) )
			goodFile = true;

	return !goodFile;
}

/* _________________________________________________________________________

	During standard file null events, check to see if the user has changed
	the current selection.  If so, we need to update the profile name
	string in the window.  If nothing is selected, the string will end
	up blank.
   _________________________________________________________________________ */

static pascal short TheDialogHook ( short item, DialogRef theDialogRef, FilterDataStructure *fds )
{

	if (item == sfHookFirstCall)
	{	
		unsigned long	refcon;
		short			csVRefNum;
		long			csDirID;
		
		refcon = GetWRefCon((WindowRef)theDialogRef);
		if ( refcon == sfMainDialogRefCon )
		{
			if ( CMGetColorSyncFolderSpec(kOnSystemDisk, false, &csVRefNum, &csDirID) == noErr )
			{
				fds->reply.sfFile.vRefNum = csVRefNum;
				fds->reply.sfFile.parID = csDirID;
				fds->reply.sfFile.name[0] = 0;
				return sfHookChangeSelection;
			}
		}
		if (refcon == sfLockWarnDialogRefCon)
			return 1;	// simulate hit of OK button
	}

	if (item == sfHookNullEvent)
	{
		OSErr		err;
		Str255		profName;
		FSSpec		*oldSpec = &fds->lastSpec;
		FSSpec		*newSpec = &fds->reply.sfFile;
		FInfo		fndrInfo;

		// First compare the new spec with the last one we looked at.
		
		if ( (newSpec->vRefNum != oldSpec->vRefNum) ||
			 (newSpec->parID != oldSpec->parID) ||
			 !EqualString(newSpec->name,oldSpec->name,true,true) )
		{	// OK, they are different.  Save the new one.
				
			*oldSpec = *newSpec;
			
			//	Get the Info string.  Note that if newSpec is not a valid
			//  filespec (as will be the case when nothing is selected),
			//	we end up with an empty string.
			
			err = FSpGetFInfo(oldSpec,&fndrInfo);
			if (!err && (fndrInfo.fdType == kProfileType) )
				CSProfileInfoString(oldSpec,profName);
			else
				profName[0] = 0;
				
			*oldSpec =*newSpec;  // save oldSpec again in case above changed it
			
			{
				Rect r;
				short itemType;
				Handle textHdl;
				
				GetDialogItem (theDialogRef, itemSFProfileName, &itemType, &textHdl, &r);
				SetDialogItemText (textHdl, profName);
			}
		}
	}
	
	return item;
}

static OSErr ReadProfileCM2Header( FSSpec *spec, CM2Header *hdr)
{
	OSErr			err=noErr;
	short			refNum;
	long			readSize;
	
	refNum = 0;
	readSize = sizeof(CM2Header);
	
	if (!spec || !hdr)
		err = paramErr;

	if (!err)
		err = FSpOpenDF( spec, fsRdPerm, &refNum);

	if (!err)
		err = FSRead( refNum, &readSize, (Ptr)hdr);
	
	if (refNum)
		(void) FSClose(refNum);
	
	return err;
}

/* _________________________________________________________________________

	Given a filespec for a profile, open it and get the profile name from
	the text description tag.  If spec==nil, get the string from the system
	profile.
	
	spg -- not localized?
	spg -- use friggin script code....
   _________________________________________________________________________ */
   
static void CSProfileInfoString ( FSSpec *spec, Str255 info )
{
	CMProfileRef prof;
	CMError err;
	
	info[0]=0;				// clear string first just to be safe

	if (spec)
	{
		CMProfileLocation location;
		location.locType = cmFileBasedProfile;
		location.u.fileLoc.spec = *spec;
		err = CMOpenProfile(&prof,&location);
	}
	else
		err = CMGetSystemProfile(&prof);
	
	if (!err)
	{
		ScriptCode script;
		err = CMGetScriptProfileDescription(prof, info, &script);	// in ProfileConvert.c
		CMCloseProfile(prof);
	}
}

/* _________________________________________________________________________

	Jam the low-mem globals for StandardFile such that the dialog comes up
	in the specified directory.
   _________________________________________________________________________ */
   
static void SetSFDefaults ( short vRefNum, long dirID )
{
	LMSetSFSaveDisk(-1 * vRefNum);
	LMSetCurDirStore(dirID);
}


/* _________________________________________________________________________

	Get the low-mem globals for the directory that the StandardFile dialog 
	comes up in.
   _________________________________________________________________________ */
   
static void GetSFDefaults ( short *vRefNum, long *dirID )
{
	*vRefNum = -1 * LMGetSFSaveDisk();
	*dirID = LMGetCurDirStore();
}


static Fixed BestGammaForBytes( int count, UInt8 *s)
{
	double	g;
	UInt32	i;
	double	range = (double)(s[count-1] - s[0]);
	UInt32	sum = 0;
	
	for (i=0; i<count; i++)
		sum += s[i]-s[0];
	sum -= (s[count-1]-s[0])/2;
	
	g = ( (range*(count-1)) / ((double)sum) ) - 1.0;
	
	return DoubToFixed(g);
}


static Fixed BestGammaForShorts( int count, UInt16 *s)
{
	double	g;
	UInt32	i;
	double	range = (double)(s[count-1] - s[0]);
	
	UInt32	sum = 0;
	
	for (i=0; i<count; i++)
		sum += s[i]-s[0];
	sum -= (s[count-1]-s[0])/2;
	
	g = ( (range*(count-1)) / ((double)sum) ) - 1.0;
	
	return DoubToFixed(g);
}


#pragma mark -


static char* Pstrcpy2C (char *dst, ConstStr255Param src, const unsigned char dstLen )
{
	unsigned char bytesToCopy ;
	
	bytesToCopy = *src ;
	if (bytesToCopy > dstLen)					// make sure were not too big
		bytesToCopy = dstLen ;
		
	BlockMoveData(	src + 1, dst,
					bytesToCopy ) ;				// copy string in
	dst[bytesToCopy] = 0 ;						// terminate c-string
	
	return dst ;
}

static StringPtr Pstrcpy (StringPtr dst, ConstStr255Param src, const unsigned char dstLen )
{
	unsigned char bytesToCopy ;
	
	bytesToCopy = *src ;
	if (bytesToCopy > dstLen)					// make sure were not too big
		bytesToCopy = dstLen ;
		
	BlockMoveData(	src + 1, dst + 1,
					bytesToCopy ) ;				// copy string in
	*dst = bytesToCopy ;						// adjust length byte
	
	return dst ;
}

