
//===============================================================================
//
//								SimpleCalibratorEvents.c
//
//===============================================================================


#include "SimpleCalibratorEvents.h"


#define kEnterKeyASCII			0x03
#define kTabKeyASCII			0x09
#define	kReturnKeyASCII			0x0D
#define	kEscapeKeyASCII			0x1B

#include <Sound.h>
#include <Dialogs.h>
#include <Controls.h>
#include <fp.h>			// For pow() and log()


static void GetDialogItemValue (DialogRef, SInt16, SInt16 *);
static void HandleDialogEvent (EventRecord *, CalibratorGlobals *);
static void DoMouseDownEvent (CalibratorGlobals *, EventRecord *);
static void HandleNonDialogEvent (EventRecord *, CalibratorGlobals *);
static void FlashDialogButton (DialogRef, SInt16);


//===================================================================== Functions
//---------------------------------------------------------------------  GetDialogItemValue
// Returns the value or "state" of a dialog item.  For checkboxes and
// radio buttons, this may be a 1 or 0.

static void GetDialogItemValue (DialogRef theDialog, SInt16 item, SInt16 *theState)
{
	Rect		itemRect;
	Handle		itemHandle;
	SInt16		itemType;
	
	GetDialogItem(theDialog, item, &itemType, &itemHandle, &itemRect);
	*theState = GetControlValue((ControlRef)itemHandle);
}

//--------------------------------------------------------------------- DrawDefaultButtonOutline

pascal void DrawDefaultButtonOutline (DialogRef theDialog, SInt16 theItem)
{
	Rect		itemBounds;
	Handle		itemHandle;
	SInt16		itemType;
	
	GetDialogItem(theDialog, theItem, &itemType, &itemHandle, &itemBounds);
	SetPort(theDialog);
	PenSize(3, 3);
	FrameRoundRect(&itemBounds, 17, 17);
	PenNormal();
}

//--------------------------------------------------------------------- DrawGammaTable

pascal void DrawGammaTable (DialogRef theDialog, SInt16 theItem)
{
	CalibratorGlobals	*theGlobals;
	Rect				itemBounds;
	double				x, y;
	Handle				itemHandle;
	SInt16				itemType;
	SInt16				wide;
	SInt16				i;
	
	theGlobals = (CalibratorGlobals *)GetWRefCon(theDialog);
	
	GetDialogItem(theDialog, theItem, &itemType, &itemHandle, &itemBounds);
	SetPort(theDialog);
	
	wide = itemBounds.right - itemBounds.left;
	for (i = 0; i < wide; i++)
	{
		x = (double)i / (double)wide;
		y = pow(x, theGlobals->theNewGamma.red) * wide;
		
		ForeColor(cyanColor);
		MoveTo(itemBounds.left + i, itemBounds.bottom - 1);
		Line(0, -floor(y));
		
		ForeColor(whiteColor);
		LineTo(itemBounds.left + i, itemBounds.top);
	}
	
	ForeColor(blackColor);
	FrameRect(&itemBounds);
	
	PenNormal();
}

//--------------------------------------------------------------------- MainEventLoop

void MainEventLoop (CalibratorGlobals *theGlobals)
{
	EventRecord	theEvent;
	SInt32		sleep;
	
	sleep = 2;
	if (WaitNextEvent(everyEvent, &theEvent, sleep, 0L))
	{		
		if (IsDialogEvent(&theEvent))
		{
			if (theEvent.what == keyDown)
			{
				if (((theEvent.message & charCodeMask) == kEnterKeyASCII) || 
						((theEvent.message & charCodeMask) == kReturnKeyASCII))
				{
					FlashDialogButton(theGlobals->mainWindow, 1);
					theGlobals->quitting = true;
					theGlobals->createIt = true;
					return;
				}
				else if (((theEvent.message & charCodeMask) == kEscapeKeyASCII) || 
						(((theEvent.message & charCodeMask) == '.') && 
						((theEvent.modifiers & cmdKey) != 0)))
				{
					FlashDialogButton(theGlobals->mainWindow, 2);
					theGlobals->quitting = true;
					theGlobals->createIt = false;
					return;
				}
			}
			
				// It seems update events (even for other windows) get swallowed
				// by DialogSelect().  We want to check for these explicitly.
			if ((theEvent.what == updateEvt) || (theEvent.what == activateEvt))
			{
				if ((WindowRef)theEvent.message != theGlobals->mainWindow)
				{
					if (theGlobals->eventProc)
						CallCalibrateEventProc(theGlobals->eventProc, &theEvent);
					return;
				}
			}
			
			HandleDialogEvent(&theEvent, theGlobals);
		}
		else
			HandleNonDialogEvent(&theEvent, theGlobals);
	}
	else
	{
		DialogPeek	thePeek;
		
				// Allow the text edit caret to blink.
		thePeek = (DialogPeek)theGlobals->mainWindow;
		TEIdle(thePeek->textH);
	}
}

//--------------------------------------------------------------------- HandleDialogEvent

static void HandleDialogEvent (EventRecord *theEvent, CalibratorGlobals *theGlobals)
{
	Rect		itemBounds;
	DialogRef	theDialog;
	Handle		itemHandle;
	OSErr		theErr;
	SInt16		itemHit;
	SInt16		itemValue;
	SInt16		itemType;
	
				// See if we can handle it.
	if (!DialogSelect(theEvent, &theDialog, &itemHit))
		return;
	
	switch (itemHit)
	{
		case 1:		// Okay button
		theGlobals->quitting = true;
		theGlobals->createIt = true;
		break;
		
		case 2:		// Cancel button
		theGlobals->quitting = true;
		theGlobals->createIt = false;
		break;
		
		case 9:		// Gamma power
		GetDialogItemValue(theDialog, itemHit, &itemValue);
		theGlobals->theNewGamma.red = (double)itemValue / 25.0;
		theGlobals->theNewGamma.green = theGlobals->theNewGamma.red;
		theGlobals->theNewGamma.blue = theGlobals->theNewGamma.red;
		theErr = VPTProgramGammaExponent(theGlobals->displayID, &(theGlobals->theNewGamma));
		
		SetPort(theGlobals->mainWindow);
		GetDialogItem(theGlobals->mainWindow, 4, &itemType, &itemHandle, &itemBounds);
		InvalRect(&itemBounds);
		break;
		
		default:
		break;
	}
}

//--------------------------------------------------------------------- DoMouseDownEvent

static void DoMouseDownEvent (CalibratorGlobals *theGlobals, EventRecord *theEvent)
{
	Rect		dragBounds;
	WindowRef	whichWindow;
	Point		mousePt;
	Point		localMouse;
	SInt16		theWindowPart;
	
	mousePt = theEvent->where;
	theWindowPart = FindWindow(mousePt, &whichWindow);
	if (whichWindow != theGlobals->mainWindow)
		return;
	
	switch (theWindowPart)
	{
				// We only want to drag a window when we have an 'eventProc' 
				// provided to us for Monitors and Sound.  Otherwise, our 
				// drag will generate update events for M&S and we won't be 
				// able to facilitate them with a mechanism for redrawing their
				// content.
		case inDrag:
		if (theGlobals->eventProc)
		{
				// We keep the drag confined to the monitor we were asked to 
				// calibrate.
			dragBounds = theGlobals->displayBounds;
			dragBounds.top += 20;
			dragBounds.right -= 240;
			dragBounds.bottom -= 200;
			localMouse = mousePt;
			GlobalToLocal(&localMouse);
			OffsetRect(&dragBounds, localMouse.h, localMouse.v);
			DragWindow(whichWindow, mousePt, &dragBounds);
		}
		break;
		
		case inContent:
		break;
		
		case inGoAway:
		SetPort(whichWindow);
		if (TrackGoAway(whichWindow, mousePt))
			theGlobals->quitting = true;
		break;
		
		default:
		SysBeep(10);
		break;
	}
}

//--------------------------------------------------------------------- HandleNonDialogEvent

static void HandleNonDialogEvent (EventRecord *theEvent, CalibratorGlobals *theGlobals)
{
	switch (theEvent->what)
	{
		case mouseDown:
		DoMouseDownEvent(theGlobals, theEvent);
		break;
		
		case keyDown:
		case autoKey:
		SysBeep(10);
		break;
		
				// It is only on update and activate events that you need to
				// call through to 'CallCalibrateEventProc' (and pass the event
				// back to the Monitors and Sound control panel. 
		case updateEvt:
		case activateEvt:
		if ((WindowRef)theEvent->message != theGlobals->mainWindow)
		{
				// IMPORTANT!!!  Make sure 'eventProc' is not NIL before you call!
			if (theGlobals->eventProc)
				CallCalibrateEventProc(theGlobals->eventProc, theEvent);
		}
		break;
	}
}

//-------------------------------------------------------------- FlashDialogButton
// Flashes the default dialog button (item = 1) so as to make it appear
// as though the user clicked on it.

static void FlashDialogButton (DialogRef theDialog, SInt16 itemNumber)
{
	Rect		itemRect;
	Handle		itemHandle;
	UInt32		dummyLong;
	SInt16		itemType;
	
	GetDialogItem(theDialog, itemNumber, &itemType, &itemHandle, &itemRect);
	HiliteControl((ControlHandle)itemHandle, kControlButtonPart);
	Delay(8, &dummyLong);
	HiliteControl((ControlHandle)itemHandle, 0);
}

