
//===============================================================================
//
//							  SimpleCalibratorUtils.c
//
//===============================================================================


#include <Fonts.h>
#include <Resources.h>
#include "SimpleCalibratorUtils.h"


//===================================================================== Functions
//--------------------------------------------------------------------- CenterDialogInRect

void CenterDialogInRect (SInt16 dialogID, Rect *parentBounds)
{
	DialogTHndl	dlogHandle;
	Rect		theScreen, dlogBounds;
	SInt16		hPos, vPos;
	Byte		wasState;
	
	theScreen = *parentBounds;
	dlogHandle = (DialogTHndl)GetResource('DLOG', dialogID);
	if (dlogHandle)
	{
		wasState = HGetState((Handle)dlogHandle);
		HLock((Handle)dlogHandle);
		
		dlogBounds = (**dlogHandle).boundsRect;
		OffsetRect(&dlogBounds, -dlogBounds.left, -dlogBounds.top);
		
		hPos = ((theScreen.right - theScreen.left) - dlogBounds.right) / 2;
		vPos = ((theScreen.bottom - theScreen.top) - dlogBounds.bottom) / 3;
		
		OffsetRect(&dlogBounds, theScreen.left + hPos, theScreen.top + vPos);
		
		(**dlogHandle).boundsRect = dlogBounds;
		HSetState((Handle)dlogHandle, wasState);
	}
}

//--------------------------------------------------------------------- PasStringCopy

void PasStringCopy (StringPtr p1, StringPtr p2)
{
	SInt16		stringLength;
	
	stringLength = *p2++ = *p1++;
	while (--stringLength >= 0)
		*p2++ = *p1++;
}

//--------------------------------------------------------------------- Pstrcpy2C

char * Pstrcpy2C (char *dst, ConstStr255Param src, const UInt8 dstLen)
{
	UInt8		bytesToCopy;
	
	bytesToCopy = *src;
	
				// Make sure we're not too big.
	if (bytesToCopy > dstLen)
		bytesToCopy = dstLen;
	
				// Copy string in.
	BlockMoveData(src + 1, dst, bytesToCopy);
	
				// Terminate C-string.
	dst[bytesToCopy] = 0;
	
	return dst;
}

//--------------------------------------------------------------------- AppendMenuNoMeta

SInt16 AppendMenuNoMeta (MenuRef menu, ConstStr255Param data)
{
	UInt32		dummyMenuItemAsLong;
	SInt16		index;
	
	dummyMenuItemAsLong = 0x0364756D;
	
	AppendMenu(menu, (StringPtr)&dummyMenuItemAsLong);
	index = CountMItems(menu);
	SetMenuItemText(menu, index, data);
	
	return index;
}

//--------------------------------------------------------------------- SetScriptProfileDescription

CMError SetScriptProfileDescription (CMProfileRef prof, Str255 name, ScriptCode code)
{
	CMError		err;
	Ptr			tag;
	UInt32		offset;
	UInt32		nameLen;
	
	offset = 0;
	nameLen = name[0] + 1;
	tag = NewPtrClear(23 + 2 * nameLen);
	if (!tag)
		return memFullErr;
	
				// OSType  tag.typeDescriptor
	*((OSType *)(tag + offset)) = cmSigProfileDescriptionType;
	offset += 4;
	
				// UInt32  tag.reserved
	offset += 4;
	
				// UInt32  tag.ASCIICount
	*((UInt32 *)(tag+offset)) = nameLen;
	offset += 4;
	
				// tag.ASCIIName
	Pstrcpy2C(tag + offset, name, 255);
	offset += nameLen;
	
				// tag.UniCodeCode, UniCodeCount, UniCodeName
	offset += 8;
	
				// tag.ScriptCodeCode
	*((SInt16 *)(tag + offset)) = code;
	offset += 2;
	
				// tag.ScriptCodeCount
	*((UInt8 *)(tag + offset)) = nameLen;
	offset += 1;
	
				// tag.ScriptCodeName
	Pstrcpy2C(tag + offset, name, 255 );
	
	err = CMSetProfileElement(prof, cmSigProfileDescriptionType, GetPtrSize(tag), tag);
	
	DisposePtr(tag);
	
	return err;
}

//--------------------------------------------------------------------- InitToolbox

void InitToolbox (void)
{
	MoreMasters();
	MaxApplZone();
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	FlushEvents (everyEvent, 0);
	InitCursor();
	
	GetDateTime((unsigned long *)&qd.randSeed);
}

