/*
	File:		Calibrator.h

	Contains:	Calls to the calibrator plug-ins.
	Version:	ColorSync 2.5 or more recent
	Copyright:	 1997 by Apple Computer, Inc., all rights reserved.
*/

#ifndef __CALIBRATOR__
#define __CALIBRATOR__

#ifndef __CMAPPLICATION__
#include <CMApplication.h>
#endif
#ifndef __DISPLAYS__
#include <Displays.h>
#endif
#ifndef __ERRORS__
#include <Errors.h>
#endif
#ifndef __TYPES__
#include <Types.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

typedef pascal void (*CalibrateEventProcPtr)(EventRecord *theEvent);

#if GENERATINGCFM
typedef UniversalProcPtr CalibrateEventUPP;
#else
typedef CalibrateEventProcPtr CalibrateEventUPP;
#endif

#if GENERATINGCFM
#define NewCalibrateEventProc(userRoutine)		\
		(CalibrateEventUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppCalibrateEventProcInfo, GetCurrentArchitecture())
#else
#define NewCalibrateEventProc(userRoutine)		\
		((CalibrateEventUPP) (userRoutine))
#endif

/*
#if GENERATINGCFM
#define CallCalibrateEventProc(userRoutine, theEvent)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppCalibrateEventProcInfo, (theEvent))
#else
#endif
*/

struct CalibratorInfo {
	AVIDType 						displayID;
	CMProfileLocation 				profileLocation;
	CalibrateEventUPP 				eventProc;
	UInt32 							reserved;
	UInt32 							flags;
	Boolean 						isGood;
	SInt8 							byteFiller;
};
typedef struct CalibratorInfo CalibratorInfo;

typedef pascal Boolean (*CanCalibrateProcPtr)(AVIDType displayID);
typedef pascal OSErr (*CalibrateProcPtr)(CalibratorInfo *theInfo);

#if GENERATINGCFM
typedef UniversalProcPtr CanCalibrateUPP;
typedef UniversalProcPtr CalibrateUPP;
#else
typedef CanCalibrateProcPtr CanCalibrateUPP;
typedef CalibrateProcPtr CalibrateUPP;
#endif

enum {
	uppCalibrateEventProcInfo = kPascalStackBased
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(EventRecord *))),
	uppCanCalibrateProcInfo = kPascalStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(Boolean)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(AVIDType))),
	uppCalibrateProcInfo = kPascalStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(OSErr)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(CalibratorInfo *)))
};

#if GENERATINGCFM
#define NewCalibrateEventProc(userRoutine)		\
		(CalibrateEventUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppCalibrateEventProcInfo, GetCurrentArchitecture())
#define NewCanCalibrateProc(userRoutine)		\
		(CanCalibrateUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppCanCalibrateProcInfo, GetCurrentArchitecture())
#define NewCalibrateProc(userRoutine)		\
		(CalibrateUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppCalibrateProcInfo, GetCurrentArchitecture())
#else
#define NewCalibrateEventProc(userRoutine)		\
		((CalibrateEventUPP) (userRoutine))
#define NewCanCalibrateProc(userRoutine)		\
		((CanCalibrateUPP) (userRoutine))
#define NewCalibrateProc(userRoutine)		\
		((CalibrateUPP) (userRoutine))
#endif

#if GENERATINGCFM
#define CallCalibrateEventProc(userRoutine, event)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppCalibrateEventProcInfo, (event))
#define CallCanCalibrateProc(userRoutine, displayID)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppCanCalibrateProcInfo, (displayID))
#define CallCalibrateProc(userRoutine, theInfo)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppCalibrateProcInfo, (theInfo))
#else
#define CallCalibrateEventProc(userRoutine, event)		\
		(*(userRoutine))((event))
#define CallCanCalibrateProc(userRoutine, displayID)		\
		(*(userRoutine))((displayID))
#define CallCalibrateProc(userRoutine, theInfo)		\
		(*(userRoutine))((theInfo))
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#ifdef __cplusplus
}
#endif

#endif /* __CALIBRATOR__ */

