
//===============================================================================
//
//									GammaUtils.c
//
//===============================================================================


#include <Devices.h>
#include <fp.h>			// For pow() and log()
#include "GammaUtils.h"


//===================================================================== Functions
//--------------------------------------------------------------------- VPTProgramGammaExponent

OSErr VPTProgramGammaExponent (AVIDType avID, RGBGammaExponentPtr gammaExp)
{
	VPTGammaPtr		gamma;
	RGBGammaPoint	maxScale;
	OSErr			theErr;
	
	gamma = 0L;
	theErr = noErr;
	
				// Allocate some memory.
	gamma = (VPTGammaPtr)NewPtr(sizeof(VPTGamma));
	if (!gamma)
	{
		theErr = memFullErr;
		goto bail;
	}
	
				// Set gamma to desired exponent.
	maxScale.red = maxScale.green = maxScale.blue = kGammaMaxVal;
	MakeArbitraryGamma(gamma, gammaExp, &maxScale, 0L, 1);
	theErr = VPTProgramSystemGamma(avID, gamma, 0L);
	if (theErr)
		goto bail;
	
bail:
	
	if (gamma)
		DisposePtr((Ptr)gamma);
	
	return theErr;
}

//--------------------------------------------------------------------- VPTProgramSystemGamma

OSErr VPTProgramSystemGamma (AVIDType avID, VPTGamma* gamma, VptSystemGamma *optionalGamma)
{
	VptSystemGamma		systemGamma;
	VDGammaRecord		gammaRecord;
	GDHandle			gDevice;
	PixMapHandle		thePixMap;
	CTabHandle			theColorTable;
	VDSetEntryRecord	theSetEntryRec;
	Ptr					stupidPtr;
	OSErr				theErr;
	SInt16				index;
	SInt16				refNum;
	
	theErr = noErr;
	thePixMap = NULL;
	theColorTable = NULL;
	
				// Set up our own gamma table header.
	systemGamma.gVersion = 0;						// Has anyone defined gamma versions?
	systemGamma.gType = 0;							// Has anyone defined gamma types?
	systemGamma.gFormulaSize = 0;					// Don't need no stinkin' formulas!
	systemGamma.gChanCnt = 3;						// Number of channels 
	systemGamma.gDataCnt = kFullGammaX + 1;			// Size of each channel
	systemGamma.gDataWidth = 8;						// Bits per entry, bad assumption
	
	for (index = 0; index <= kGammaMaxVal; index++)
	{
		systemGamma.red[index] = gamma->red[index];
		systemGamma.green[index] = gamma->green[index];
		systemGamma.blue[index] = gamma->blue[index];
	}
	
				// Get the GDevice where we belong.
	theErr = DMGetGDeviceByDisplayID( avID, &gDevice, false);
	if (theErr)
		goto bail;
	
	refNum = (**gDevice).gdRefNum;
	
				// Set up & whack the gamma table.
	gammaRecord.csGTable = (Ptr)&systemGamma;
	stupidPtr = (Ptr)&gammaRecord;
	theErr = Control(refNum, cscSetGamma, &stupidPtr);
	if (theErr)
		goto bail;
	
				// If gamma pointer passed in, give them back a copy.
	if (optionalGamma)
		*optionalGamma = systemGamma;
	
				// Do SetEntries() to make it happen on < 16 bit video.
	thePixMap = (**gDevice).gdPMap;
	theColorTable = (**thePixMap).pmTable;
	theSetEntryRec.csTable = (ColorSpec *)(**theColorTable).ctTable;
	theSetEntryRec.csStart = 0;
	theSetEntryRec.csCount = (**theColorTable).ctSize;
	
				// On direct video devices (bit depths of 16 and 32) there is no need
				// to do a SetEntries() call. Bit depths of 8 and less DO require a
				// SetEntries call in order to take effect. 
				// Rather than checking the bit depth, we'll just go ahead and call
				// SetEntries. Direct devices will return an error when you do this, 
				// but that's ok -- there's no ill effect from it. We just have to be
				// careful NOT to check the error return value and bomb out when it
				// returns an error.
	MyVDControl(refNum, cscSetEntries, (Ptr)&theSetEntryRec);
	
				// After setting up the gamma, we need to call Display Manager in order
				// for the screen settings in effect at this moment to be saved somewhere.
				// Hope this actually works...	
	theErr = DMSaveScreenPrefs(0, 0, 0);
	if (theErr)
		goto bail;
	
bail:
	
	return theErr;
}

//--------------------------------------------------------------------- MyVDControl

OSErr MyVDControl (SInt16 refNum, SInt16 csCode, Ptr theCtrlParm)
{
	CntrlParam	myCntrlRec;
	
	myCntrlRec.ioCompletion = 0L;						// Synchronous Call
	myCntrlRec.ioVRefNum = 0;							// We don't know/care what volume
	*(VDPageInfo**) &(myCntrlRec.csParam) = (VDPageInfo*) theCtrlParm;	// Point to our control record
	
	myCntrlRec.ioCRefNum = refNum;						// Tell them which driver
	myCntrlRec.csCode = csCode;							// Tell them which call
	
				// For no apparrant reason, PBControl always returns an error, even when
				// everything works right. We'll ignore the error.		-jmd 5/13/97
	PBControl((ParmBlkPtr) &myCntrlRec,false);			// make a synchronous call
	
	return noErr;
}

//--------------------------------------------------------------------- MakeArbitraryGamma
// This routine, given exponents for each of R, G, and B, will generate 
// a gamma curve scaled to the level given by the RGBGammaPoint variable.

OSErr MakeArbitraryGamma (VPTGamma *gamma, RGBGammaExponent *exponent, 
		RGBGammaPoint *scale, RGBGammaPoint *offset, SInt16 stepValue)
{
	double*		redCurve;
	double*		greenCurve;
	double*		blueCurve;
	double		temp;
	SInt16		index;
	OSErr		theErr;
	
	redCurve = 0L;
	greenCurve = 0L;
	blueCurve = 0L;
	theErr = noErr;
	
				// Allocate memory for the three curves.
	redCurve = (double *)NewPtr(256 * sizeof(double));
	greenCurve = (double *)NewPtr(256 * sizeof(double));
	blueCurve = (double *)NewPtr(256 * sizeof(double));
	if (!redCurve || !greenCurve || !blueCurve)
	{
		theErr = memFullErr;
		goto bail;
	}
	
				// Calculate exponent curve.
	for (index = 0; index <= kGammaMaxVal; index += stepValue)
	{
		redCurve[index] = pow((double)index, exponent->red);
		greenCurve[index] = pow((double)index, exponent->green);
		blueCurve[index] = pow((double)index, exponent->blue);
	}
	
				// Scale it down.
	for (index = 0; index <= kGammaMaxVal; index += stepValue)
	{
				// Red.
		if (offset)
			temp = (redCurve[index] / redCurve[kGammaMaxVal]) * (scale->red - offset->red) + offset->red;
		else
			temp = (redCurve[index] / redCurve[kGammaMaxVal]) * scale->red;
		gamma->red[index] = temp + kIntegerRoundOff;
		
				// Green.
		if (offset)
			temp = (greenCurve[index] / greenCurve[kGammaMaxVal]) * (scale->green - offset->green) + offset->green;
		else
			temp = (greenCurve[index] / greenCurve[kGammaMaxVal]) * scale->green;
		gamma->green[index] = temp + kIntegerRoundOff;
		
				// Blue.
		if (offset)
			temp = (blueCurve[index] / blueCurve[kGammaMaxVal]) * (scale->blue - offset->blue) + offset->blue;
		else
			temp = (blueCurve[index] / blueCurve[kGammaMaxVal]) * scale->blue;
		gamma->blue[index] = temp + kIntegerRoundOff;
	}
	
bail:
	
	if (redCurve)
		DisposePtr((Ptr)redCurve);
	if (greenCurve)
		DisposePtr((Ptr)greenCurve);
	if (blueCurve)
		DisposePtr((Ptr)blueCurve);
	
	return theErr;
}

