
//===============================================================================
//
//								WindowCalibratorEvents.c
//
//===============================================================================


#include "WindowCalibratorEvents.h"
#include "WindowCalibratorUtils.h"


#define kEnterKeyASCII			0x03
#define kTabKeyASCII			0x09
#define	kReturnKeyASCII			0x0D
#define	kEscapeKeyASCII			0x1B


#include <Sound.h>
#include <fp.h>			// For pow() and log()


static void DrawGammaTable (CalibratorGlobals *theGlobals, Rect *itemBounds);
static void DoMouseDownEvent (CalibratorGlobals *, EventRecord *);


//===================================================================== Functions
//--------------------------------------------------------------------- DrawGammaTable

static void DrawGammaTable (CalibratorGlobals *theGlobals, Rect *itemBounds)
{
	double				x, y;
	SInt16				wide;
	SInt16				i;
	
	wide = itemBounds->right - itemBounds->left;
	for (i = 0; i < wide; i++)
	{
		x = (double)i / (double)wide;
		y = pow(x, theGlobals->theNewGamma.red) * wide;
		
		ForeColor(cyanColor);
		MoveTo(itemBounds->left + i, itemBounds->bottom - 1);
		Line(0, -floor(y));
		
		ForeColor(whiteColor);
		LineTo(itemBounds->left + i, itemBounds->top);
	}
	
	ForeColor(blackColor);
	FrameRect(itemBounds);
	
	PenNormal();
}

//--------------------------------------------------------------------- MainEventLoop

void MainEventLoop (CalibratorGlobals *theGlobals)
{
	EventRecord	theEvent;
	SInt32		sleep;
	
	sleep = 2;
	if (WaitNextEvent(everyEvent, &theEvent, sleep, 0L))
	{
		switch (theEvent.what)
		{
			case mouseDown:
			DoMouseDownEvent(theGlobals, &theEvent);
			break;
			
			case keyDown:
			case autoKey:
			if (((theEvent.message & charCodeMask) == kEnterKeyASCII) || 
					((theEvent.message & charCodeMask) == kReturnKeyASCII))
			{
				FlashControl(theGlobals->okayButton);
				theGlobals->quitting = true;
				theGlobals->createIt = true;
			}
			else if (((theEvent.message & charCodeMask) == kEscapeKeyASCII) || 
					(((theEvent.message & charCodeMask) == '.') && 
					((theEvent.modifiers & cmdKey) != 0)))
			{
				FlashControl(theGlobals->cancelButton);
				theGlobals->quitting = true;
				theGlobals->createIt = false;
			}
			break;
			
				// It is only on update and activate events that you need to
				// call through to 'CallCalibrateEventProc' (and pass the event
				// back to the Monitors and Sound control panel. 
			case updateEvt:
			case activateEvt:
			if ((WindowRef)theEvent.message != theGlobals->mainWindow)
			{
				// IMPORTANT!!!  Make sure 'eventProc' is not NIL before you call!
				if (theGlobals->eventProc)
					CallCalibrateEventProc(theGlobals->eventProc, &theEvent);
			}
			else
			{
				Rect		destRect;
				PicHandle	thePict;
				CIconHandle	theIcon;
				
				SetPort(theGlobals->mainWindow);
				BeginUpdate(theGlobals->mainWindow);
				
				TextFont(0);
				MoveTo(8, 16 + 12);
				DrawString("\pWindow Calibrator");
				
				MoveTo(8, 66 + 12);
				DrawString("\pProfile:");
				
				theIcon = GetCIcon(128);
				SetRect(&destRect, 240, 8, 272, 40);
				PlotCIcon(&destRect, theIcon);
				DisposeCIcon(theIcon);
				
				thePict = GetPicture(128);
				SetRect(&destRect, 8, 42, 272, 62);
				DrawPicture(thePict, &destRect);
				
				thePict = GetPicture(129);
				SetRect(&destRect, 8, 91, 272, 111);
				DrawPicture(thePict, &destRect);
				
				thePict = GetPicture(130);
				SetRect(&destRect, 8, 184, 272, 204);
				DrawPicture(thePict, &destRect);
				
				SetRect(&destRect, 98, 115, 162, 179);
				DrawGammaTable(theGlobals, &destRect);
				
				DrawControls(theGlobals->mainWindow);
				DrawDefaultControlButton(theGlobals->okayButton);
				
				EndUpdate(theGlobals->mainWindow);
			}
			
			break;
		}
	}
	else
	{
//		TEIdle(thePeek->textH);
	}
}

//--------------------------------------------------------------------- DoMouseDownEvent

static void DoMouseDownEvent (CalibratorGlobals *theGlobals, EventRecord *theEvent)
{
	Rect		dragBounds;
	WindowRef	whichWindow;
	ControlRef	whichControl;
	Point		mousePt, localPt;
	Point		localMouse;
	SInt16		theWindowPart;
	SInt16		whichPart;
	
	mousePt = theEvent->where;
	localPt = mousePt;
	GlobalToLocal(&localPt);
	
	theWindowPart = FindWindow(mousePt, &whichWindow);
	if (whichWindow != theGlobals->mainWindow)
		return;
	
				// Handle window controls.
	whichPart = FindControl(localPt, theGlobals->mainWindow, &whichControl);
	if (whichPart != 0)
	{
		if (whichControl == theGlobals->popUpControl)
		{
			SInt16		itemValue;
			
			whichPart = TrackControl(whichControl, localPt, (void *)-1);
			if (whichPart == 0)
				return;
			
			itemValue = GetControlValue(theGlobals->popUpControl);
			theGlobals->profileLocation = theGlobals->allProfileLocations[itemValue - 1];
			return;
		}
		
		whichPart = TrackControl(whichControl, localPt, 0L);
		if (whichPart == 0)
			return;
		
		if (whichControl == theGlobals->okayButton)
		{
			theGlobals->quitting = true;
			theGlobals->createIt = true;
		}
		else if (whichControl == theGlobals->cancelButton)
		{
			theGlobals->quitting = true;
			theGlobals->createIt = false;
		}
		else if (whichControl == theGlobals->sliderControl)
		{
			Rect		destRect;
			OSErr		theErr;
			SInt16		itemValue;
			
			itemValue = GetControlValue(theGlobals->sliderControl);
			theGlobals->theNewGamma.red = (double)itemValue / 25.0;
			theGlobals->theNewGamma.green = theGlobals->theNewGamma.red;
			theGlobals->theNewGamma.blue = theGlobals->theNewGamma.red;
			theErr = VPTProgramGammaExponent(theGlobals->displayID, &(theGlobals->theNewGamma));
			
			SetRect(&destRect, 98, 115, 162, 179);
			DrawGammaTable(theGlobals, &destRect);
		}
		
		return;
	}
	
	switch (theWindowPart)
	{
				// We only want to drag a window when we have an 'eventProc' 
				// provided to us for Monitors and Sound.  Otherwise, our 
				// drag will generate update events for M&S and we won't be 
				// able to facilitate them with a mechanism for redrawing their
				// content.
		case inDrag:
		if (theGlobals->eventProc)
		{
				// We keep the drag confined to the monitor we were asked to 
				// calibrate.
			dragBounds = theGlobals->displayBounds;
			dragBounds.top += 20;
			dragBounds.right -= 240;
			dragBounds.bottom -= 200;
			localMouse = mousePt;
			GlobalToLocal(&localMouse);
			OffsetRect(&dragBounds, localMouse.h, localMouse.v);
			DragWindow(whichWindow, mousePt, &dragBounds);
		}
		break;
		
		case inContent:
		break;
		
		case inGoAway:
		SetPort(whichWindow);
		if (TrackGoAway(whichWindow, mousePt))
			theGlobals->quitting = true;
		break;
		
		default:
		SysBeep(10);
		break;
	}
}

