
//===============================================================================
//
//							  WindowCalibratorUtils.c
//
//===============================================================================


#include <Fonts.h>
#include <Resources.h>
#include "WindowCalibratorUtils.h"


//===================================================================== Functions
//-------------------------------------------------------------- GetLocalWindowRect
// Returns bounding rectangle of the specified window in local coords.
// (When you just need its width and height.)

void GetLocalWindowRect (WindowRef theWindow, Rect *bounds)
{
	if (theWindow)
	{
		SetPort(theWindow);
		*bounds = theWindow->portRect;
	}
}

//-------------------------------------------------------------- GetLocalWindowRect
// Returns bounding rectangle of the specified window in local coords.
// (When you just need its width and height.)

void GetGlobalWindowRect (WindowRef theWindow, Rect *bounds)
{
	if (theWindow)
	{
		GetLocalWindowRect(theWindow, bounds);
		LocalToGlobalRect(bounds);
	}
}

//-------------------------------------------------------------- RectWide
// Given a rectangle, this simple function returns the rect's width.

SInt16 RectWide (Rect *theRect)
{
	return (theRect->right - theRect->left);
}

//-------------------------------------------------------------- RectTall
// Given a rectangle, this simple function returns the rect's height.

SInt16 RectTall (Rect *theRect)
{
	return (theRect->bottom - theRect->top);
}

//-------------------------------------------------------------- CenterRectInRect
// Given two rectangles, this function centers the first rectangle
// within the second.  The second rect is unchanged.

void CenterRectInRect (Rect *rectA, Rect *rectB)
{
	SInt16	widthA, tallA;
	
	widthA = RectWide(rectA);
	tallA = RectTall(rectA);
	
	rectA->left = rectB->left + (RectWide(rectB) - widthA) / 2;
	rectA->right = rectA->left + widthA;
	
	rectA->top = rectB->top + (RectTall(rectB) - tallA) / 2;
	rectA->bottom = rectA->top + tallA;
}

//-------------------------------------------------------------- GlobalToLocalRect
// This function offsets a rectangle from global to local coordinates.
// The "local" coordinate system is assumed to be the current port (window).

void GlobalToLocalRect (Rect *theRect)
{
	Point			upperLeftPt;
	
	upperLeftPt.h = 0;
	upperLeftPt.v = 0;
	GlobalToLocal(&upperLeftPt);
	OffsetRect(theRect, upperLeftPt.h, upperLeftPt.v);
}

//-------------------------------------------------------------- LocalToGlobalRect
// This function offsets a rectangle from local to global coordinates.
// The "local" coordinate system is assumed to be the current port (window).

void LocalToGlobalRect (Rect *theRect)
{
	Point			upperLeftPt;
	
	upperLeftPt.h = 0;
	upperLeftPt.v = 0;
	LocalToGlobal(&upperLeftPt);
	OffsetRect(theRect, upperLeftPt.h, upperLeftPt.v);
}

//--------------------------------------------------------------------- PasStringCopy

void PasStringCopy (StringPtr p1, StringPtr p2)
{
	SInt16		stringLength;
	
	stringLength = *p2++ = *p1++;
	while (--stringLength >= 0)
		*p2++ = *p1++;
}

//--------------------------------------------------------------------- Pstrcpy2C

char * Pstrcpy2C (char *dst, ConstStr255Param src, const UInt8 dstLen)
{
	UInt8		bytesToCopy;
	
	bytesToCopy = *src;
	
				// Make sure we're not too big.
	if (bytesToCopy > dstLen)
		bytesToCopy = dstLen;
	
				// Copy string in.
	BlockMoveData(src + 1, dst, bytesToCopy);
	
				// Terminate C-string.
	dst[bytesToCopy] = 0;
	
	return dst;
}

//--------------------------------------------------------------------- AppendMenuNoMeta

SInt16 AppendMenuNoMeta (MenuRef menu, ConstStr255Param data)
{
	UInt32		dummyMenuItemAsLong;
	SInt16		index;
	
	dummyMenuItemAsLong = 0x0364756D;
	
	AppendMenu(menu, (StringPtr)&dummyMenuItemAsLong);
	index = CountMItems(menu);
	SetMenuItemText(menu, index, data);
	
	return index;
}

//--------------------------------------------------------------------- SetScriptProfileDescription

CMError SetScriptProfileDescription (CMProfileRef prof, Str255 name, ScriptCode code)
{
	CMError		err;
	Ptr			tag;
	UInt32		offset;
	UInt32		nameLen;
	
	offset = 0;
	nameLen = name[0] + 1;
	tag = NewPtrClear(23 + 2 * nameLen);
	if (!tag)
		return memFullErr;
	
				// OSType  tag.typeDescriptor
	*((OSType *)(tag + offset)) = cmSigProfileDescriptionType;
	offset += 4;
	
				// UInt32  tag.reserved
	offset += 4;
	
				// UInt32  tag.ASCIICount
	*((UInt32 *)(tag+offset)) = nameLen;
	offset += 4;
	
				// tag.ASCIIName
	Pstrcpy2C(tag + offset, name, 255);
	offset += nameLen;
	
				// tag.UniCodeCode, UniCodeCount, UniCodeName
	offset += 8;
	
				// tag.ScriptCodeCode
	*((SInt16 *)(tag + offset)) = code;
	offset += 2;
	
				// tag.ScriptCodeCount
	*((UInt8 *)(tag + offset)) = nameLen;
	offset += 1;
	
				// tag.ScriptCodeName
	Pstrcpy2C(tag + offset, name, 255 );
	
	err = CMSetProfileElement(prof, cmSigProfileDescriptionType, GetPtrSize(tag), tag);
	
	DisposePtr(tag);
	
	return err;
}

//--------------------------------------------------------------------- InitToolbox

void InitToolbox (void)
{
	MoreMasters();
	MaxApplZone();
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	FlushEvents (everyEvent, 0);
	InitCursor();
	
	GetDateTime((unsigned long *)&qd.randSeed);
}

//-------------------------------------------------------------- DrawDefaultControlButton

void DrawDefaultControlButton (ControlRef theControl)
{
	Rect		itemRect;
	
	itemRect = (*theControl)->contrlRect;
	InsetRect(&itemRect, -4, -4);
	PenSize(3, 3);
	FrameRoundRect(&itemRect, 16, 16);
	PenNormal();
}

//-------------------------------------------------------------- FlashControl

void FlashControl (ControlRef theControl)
{
	HiliteControl(theControl, kControlButtonPart);
	Delay(8, 0L);
	HiliteControl(theControl, 0);
}

