/* ___________________________________________________________________________________________

	File:		FileUtils.h

	Contains:	Header for the FileUtils.c source.
	Version:	ColorSync 2 or later
	Written by:	Steve Romero	Critical Path Software, Inc. (503) 635-9585
	Copyright:	 1993, 1995-1998 by Apple Computer, Inc. All rights reserved.

	File Ownership:
		DRI:				David Hayward
		Other Contact:		Steve Swen
		Technology:			ColorSync 2

	Writers:

		(ebb)	Eric Broadbent
		(jkc)	john calhoun
		(DH)	David Hayward

	Change History (most recent first):

		<14>	  5/5/98	ebb		Adding Set/GetEOF abstractions.
		<13>	 8/19/97	ebb		Adding cross-platform support.
		<12>	 1/30/98	DH		Added InfoCopy (used by scripting plugins).
		<11>	12/23/97	DH		Added RsrcCopy routine.
		<10>	12/17/97	DH		Fix resource fork bug
		 <9>	 9/19/97	DH		Added some new handy routines.
		 <8>	 9/16/97	DH		__FILEUTILS__ was not defined.
		 <7>	 9/15/97	DH		Added FileClose()
		 <6>	 7/21/97	DH		Added CreateTempFile.
		 <5>	  7/9/97	DH		Added stuff for C++.
		 <4>	  6/4/97	jkc		Made ResolveAliasFileMountOption() no longer static.
		 <3>	  6/7/96	DH		Added ReadPos and WritePos
		 <2>	 2/12/96	DH		Added GetFileLocation and FSpGetFileLocation
		 <1>	12/15/95	DH		First checked in.
		<4>		11/10/94	tem		MasterInterfaces name changes
		<3>		4/18/94		sjr		Moved to Support From Dispatcher:Sources.
		<2>		4/15/94		sjr		Check in for first build.
		<1>		3/15/94		sjr		Original Check in.

	To do:
	
   ___________________________________________________________________________________________ */


#ifndef __FileUtils__
#define	__FileUtils__

#include <Files.h>
#include <Finder.h>
#include <MacTypes.h>

typedef	FSSpec	FileSpec;
typedef	SInt16	FileRef;


enum
{
	kDontDelete		= 0,
	kDeleteIfExists	= 1,
	kDontCreate,
	kCreate
};

#define	kFileIsAlias		0x0001
#define	kFileIsDirectory	0x0010
#define	kIsDirectory		kFileIsDirectory
#define	kFileIsProfile		0x0020
#define	kIsAliasToDirectory	0x0040


#ifdef __cplusplus
extern "C" {
#endif

												
OSErr	FileClose					(			FileRef			refNum );

OSErr	FileGetEOF					(			FileRef			refNum,
												unsigned long	*eof );

OSErr	FileSetEOF					(			FileRef			refNum,
												unsigned long	eof );
												
OSErr	ResolveFileSpec				(			FileSpec		*fromSpec,
												FileSpec		*toSpec );

OSErr	RsrcOpenAndGetEOF			(	const	FileSpec		*spec,
												short			perm,
												FileRef			*refNum,
												long			*length );

OSErr	RsrcCopy					(	const	FileSpec		*srcSpec,
										const	FileSpec		*dstSpec );

OSErr	InfoCopy					(	const	FileSpec		*srcSpec,
										const	FileSpec		*dstSpec );

OSErr	ResolveAliasFileMountOption	(			FileSpec		*fileFSSpec,
												Boolean			resolveAliasChains,
												Boolean			*targetIsFolder, 
									  			Boolean			*wasAliased,
												Boolean			mountRemoteVols );


#ifdef __cplusplus
}
#endif

#endif

