/*
	File:		GIFcore.h
	Contains:	
	Version:	ColorSync 2 or later
	Copyright:	 1998 by Apple Computer, Inc., all rights reserved.

	File Ownership:
		DRI:				David Hayward
		Other Contact:		Steve Swen
		Technology:			ColorSync 2

	Writers:

		(DH)	David Hayward

	Change History (most recent first):

		 <1>	 3/26/98	DH		First checked in.
*/

#ifndef GIFCORE_H
#define GIFCORE_H

#include <stdio.h>
#include <CMApplication.h>


#define kGifImageSeparator			0x2C
#define kGifTrailer					0x3B

#define kGifExtensionIntroducer		0x21

#define kGifGraphicControlExtLabel	0xF9
#define kGifPlainTextExtLabel		0x01
#define kGifApplicationExtLabel		0xFF
#define kGifCommentExtLabel			0xFE

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=mac68k
#elif PRAGMA_STRUCT_PACKPUSH
	#pragma pack(push, 2)
#elif PRAGMA_STRUCT_PACK
	#pragma pack(2)
#endif

/*
**  The GIF header format.
**
**  This structure actually contains the header, logical screen
**  descriptor, and the global color table for the GIF image.
*/
typedef struct GifHeader
{
	UInt8	Signature[3];		/* ID Signature */
	UInt8	Version[3];			/* Version Number */
	UInt16	ScreenWidth;		/* Logical Screen Width */
	UInt16	ScreenHeight;		/* Logical Screen Height */ 
	UInt8	PackedField;		/* Color Information */
	UInt8	ColorIndex;			/* Background Color Index */
	UInt8	AspectRatio;		/* Pixel Aspect Ratio */
} GifHeaderRec, *GifHeaderPtr;


/*
**  The GIF Image Descriptor.
*/
typedef struct GifImageDescriptor
{
	UInt8	ImageSeparator;		/* Image Descriptor identifier (always 0x2C) */
	UInt16	ImageLeft;			/* X position of image on the display */
	UInt16	ImageTop;			/* Y position of image on the display */
	UInt16	ImageWidth;			/* Width of the image in pixels */
	UInt16	ImageHeight;		/* Height of the image in pixels */
	UInt8	PackedField;		/* Image and Color Table Data Information */
} GifImageDescRec, *GifImageDescPtr;


/*
**  GIF 89a Graphic Control Extension Block
*/
typedef struct GifGraphicControl
{
	UInt8	Introducer;			/* Extension Introducer (always 0x21) */
	UInt8	Label;				/* Extension Label (always 0xF9) */
	UInt8	BlockSize;			/* Size of Extension Block (always 0x04) */
	UInt8	PackedField;		/* Graphic Data Information */
	UInt16	DelayTime;			/* Delay time in hundredths of a second */
	UInt8	ColorIndex;			/* Transparent Color Index */
	Handle	data;				/* additional data sub-blocks */
} GifGraphicControlRec, *GifGraphicControlPtr;


/*
**  GIF 89a Plain Text Extension Block
*/
typedef struct GifPlainText
{
	UInt8	Introducer;			/* Extension Introducer (always 0x21) */
	UInt8	Label;				/* Extension Label (always 0x01) */
	UInt8	BlockSize;			/* Size of Extension Block (always 0x0C) */
	UInt16	TextGridLeft;		/* X position of text grid in pixels */
	UInt16	TextGridTop;		/* Y position of text grid in pixels */
	UInt16	TextGridWidth;		/* Width of the text grid in pixels	 */
	UInt16	TextGridHeight;		/* Height of the text grid in pixels */
	UInt8	CellWidth;			/* Width of a grid cell in pixels */
	UInt8	CellHeight;			/* Height of a grid cell in pixels */
	UInt8	TextFgColorIndex;	/* Text foreground color index value */
	UInt8	TextBgColorIndex;	/* Text background color index value */
	Handle	data;				/* additional data sub-blocks */
} GifPlainTextRec, *GifPlainTextPtr;


/*
**  GIF 89a Application Extension Block
*/
typedef struct GifApplication
{
	UInt8	Introducer;			/* Extension Introducer (always 0x21) */
	UInt8	Label;				/* Extension Label (always 0xFF) */
	UInt8	BlockSize;			/* Size of Extension Block (always 0x0B) */
	UInt8	Identifier[8];		/* Application Identifier */
	UInt8	AuthentCode[3];		/* Application Authentication Code */
	Handle	data;				/* additional data sub-blocks */
} GifApplicationRec, *GifApplicationPtr;


/*
**  GIF 89a Comment Extension Block
*/
typedef struct GifComment
{
	UInt8	Introducer;			/* Extension Introducer (always 0x21) */
	UInt8	Label;				/* Comment Label (always 0xFE) */
	Handle	data;				/* additional data sub-blocks */
} GifCommentRec, *GifCommentPtr;


#if PRAGMA_STRUCT_ALIGN
	#pragma options align=reset
#elif PRAGMA_STRUCT_PACKPUSH
	#pragma pack(pop)
#elif PRAGMA_STRUCT_PACK
	#pragma pack()
#endif

/*
**  Function Prototypes
*/

//----------------------------------------------------------------------
//
//	GIF
//
//----------------------------------------------------------------------

OSErr			GIFEmbed	(FILE *from, FILE *into, CMProfileRef embedProf);
OSErr			GIFMatch	(FILE *from, FILE *into, CMProfileRef embedProf, CMWorldRef cw);
CMError			GIFOpenEmbeddedProfile ( FILE* in, CMProfileRef *ref, UInt32 qual, UInt32 intent );

#endif	/* GIFCORE_H */
