/*
	File:		plugUtil.c

	Contains:	

	Version:	ColorSync 2 or later

	Copyright:	 1997-1998 by Apple Computer, Inc., all rights reserved.

	File Ownership:

		DRI:				David Hayward

		Other Contact:		Steve Swen

		Technology:			ColorSync 2

	Writers:

		(DH)	David Hayward

	Change History (most recent first):

		 <5>	 4/10/98	DH		Added OpenProfileHdlAsTemp.
		 <4>	 2/16/98	DH		[2213180]  Named profiles are no longer allowed.
		 <3>	 11/3/97	DH		Minor changed to OpenDefaultProfile.
		 <2>	 9/25/97	DH		Added to OpenDefaultProfile().
*/


#include "DebugAssert.h"
#include "plugUtil.h"


CMError
OpenProfileHdl ( Handle h, CMProfileRef *ref, UInt32 qual, UInt32 intent )
{
	CMError					theErr;
	CMProfileLocation		loc;
	
	theErr = noErr;
	loc.locType = cmHandleBasedProfile;
	loc.u.handleLoc.h = h;
	
	if (qual!=kProfileDefault || intent!=kProfileDefault)
	{
		CMAppleProfileHeader header;
		
		// get profile header
		header = **(CMAppleProfileHeader**)h;
		
		// Set the quality flag bits
		if (qual != kProfileDefault)
		{
			header.cm2.flags &= ~cmQualityMask;
			header.cm2.flags |= (qual << 16);
		}
			
		// Set the rendering intent
		if (intent != kProfileDefault)
			header.cm2.renderingIntent = intent;
		
		// set profile header
		**(CMAppleProfileHeader**)h = header;
	}
	
	theErr = CMOpenProfile(ref, &loc);
	require( theErr==noErr, bail );
	
bail:
	
	return theErr;
}


CMError
OpenProfileHdlAsTemp ( Handle h, CMProfileRef *ref, UInt32 qual, UInt32 intent )
{
	CMError					theErr;
	CMProfileRef			tmpProf = nil;
	CMProfileLocation		tempLoc;
	
	theErr = OpenProfileHdl( h, &tmpProf, qual, intent );
	require (theErr==noErr, bail);
	
	tempLoc.locType = cmNoProfileBase;
	theErr = CMCopyProfile(ref, &tempLoc, tmpProf);
	CMCloseProfile(tmpProf);
	require( theErr==noErr, bail);
	
bail:
	
	return theErr;
}


#define maxProfiles 3
struct CMConcatProfileSetN {
	unsigned short 		keyIndex;					// Zero-based
	unsigned short 		count;						// One-based count
	CMProfileRef 		profileSet[maxProfiles];	// Variable. Ordered from Source -> Dest
};
typedef struct CMConcatProfileSetN CMConcatProfileSetN;


CMError
PrepareColorWorld ( CMProfileRef srcProf, CMProfileRef dstProf, CMProfileRef prfProf, 
					CMWorldRef *cw )
{	
	CMConcatProfileSetN	profileList;
	OSErr 				result = noErr;
	UInt16				numProfiles = 0;
	UInt16				curProfile;
	CMWorldRef			theCW;
	
	if (srcProf)	numProfiles++;
	if (dstProf)	numProfiles++;
	if (prfProf)	numProfiles++;
	
	if (!numProfiles || !srcProf)
		result = paramErr;
	require( result==noErr, bail);
	
	profileList.keyIndex = (numProfiles>1)?(1):(0);
	profileList.count = numProfiles;
	for (curProfile = 0; curProfile < numProfiles; curProfile++)
		profileList.profileSet[curProfile] = nil;
	
	// Fill the CMConcatProfileSet with our profiles
	profileList.profileSet[0] = srcProf;
	profileList.profileSet[1] = dstProf;
	profileList.profileSet[2] = prfProf;
	
	// Create the ColorWorld
	if (numProfiles == 2)
		result = NCWNewColorWorld( &theCW, profileList.profileSet[0], profileList.profileSet[1]);
	else
		result = CWConcatColorWorld( &theCW, (CMConcatProfileSet*)&profileList);
	require( result==noErr, bail);
	*cw = theCW;
	
bail:
	
	return result;		
}

