#ifndef __DEBUGASSERT__
#define __DEBUGASSERT__

#include <MacTypes.h>



#define DEBUG_LEVEL_PRODUCTION	0
#define DEBUG_LEVEL_BREAK_ONLY	1
#define DEBUG_LEVEL_EXTERNAL	3
#define DEBUG_LEVEL_INTERNAL	4
#define DEBUGFULL				DEBUG_LEVEL_INTERNAL
#define DEBUG_NO_OPTIONS		0

#ifdef DEBUGLEVEL
#undef DEBUGLEVEL
#endif

#if DEBUG
	#if 	PRODUCTION
		#define DEBUGLEVEL DEBUG_LEVEL_PRODUCTION
	#elif	DEBUG_EXTERNAL
		#define DEBUGLEVEL DEBUG_LEVEL_EXTERNAL
	#elif	DEBUG_INTERNAL
		#define DEBUGLEVEL DEBUG_LEVEL_INTERNAL
	#elif	DEBUG_BREAK_ONLY
		#define DEBUGLEVEL DEBUG_LEVEL_BREAK_ONLY
	#else
		#define DEBUGLEVEL DEBUG_LEVEL_INTERNAL
	#endif
#endif

#ifndef	DEBUGLEVEL
#define DEBUGLEVEL DEBUG_LEVEL_PRODUCTION
#endif


#ifndef COMPONENT_SIGNATURE
#define COMPONENT_SIGNATURE '?*?*'
#endif


/*
	Options
*/


#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT
#pragma import on
#endif

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=mac68k
#elif PRAGMA_STRUCT_PACKPUSH
	#pragma pack(push, 2)
#elif PRAGMA_STRUCT_PACK
	#pragma pack(2)
#endif


enum {
	kParamError					= 0x00001000,
	kInternalCheckError			= 0x00002000,
	kCompatibilityWarning		= 0x00004000,
	kCompatibilityError			= 0x00008000,					/* old names in case anyone's using them*/
	kCompatiblityWarning		= kCompatibilityWarning,
	kCompatiblityError			= kCompatibilityError
};


#define __DEBUG_BREAK_ONLY (DEBUGLEVEL == DEBUG_LEVEL_BREAK_ONLY)
#define __DEBUGGERBREAK Debugger()
#define QuoteExceptionString(x) #x

#define notrace (StringPtr)"\p;dprintf"


/*********************************************************************

ROUTINE
	void dprintf(traceOption, format, ...)

DESCRIPTION
	dprintf is used like printf only the information is displayed in
	MacsBug. traceOption is used to specify whether the execution
	should continue after the break or not. format contains
	two types of objects: ordinary characters, which are displayed in
	Macsbug, and coversion specifications, each of which causes
	conversion and printing of the next successive argument to dprintf.
	Each conversion specification begins with a % and ends with a
	conversion character. Between % and the conversion character there
	may be, in order:
	
		Zero or more flag characters, which modify the meaning of the
		conversion specification.
		A number that specifies the minimum field width. The converted
		argument will be printed in a field at least this wide. If
		necessary it will be padded on the left (or right, if left
		adjustment is called for) to make up the field width.
		A period, which separates the field width from the precision.
		A number, the precision, that specifies the maximum number of
		characters to be printed from a string, or the number of digits
		after the decimal point of a floatingpoint value, or the minimum
		number of digits for an integer.
		An h if the integer is to be printed as a short, or l if as
		long (ignored since int is long), or L to specify that the
		following e, E, f, g, or G conversion is a long double.
	
	Note: A field width or precision may be indicated by an * instead
	of a digit string. In this case, an integer arg parameter supplies
	the field width or precision. The arg parameter that is actually
	converted is not fetched until the conversion letter is seen;
	therefore, the arg paramters specifying field width or precision
	must appear immediately before the arg parameter (if any) to be
	converted.
	
	The flag characters are:
	
		-		The result of the conversion will be left justified
				within the field.
		+		The result of a signed conversion always begins with a
				sign (+ or -).
		space	If the first character of a signed conversion is not a
				sign, a space will be prefixed to the result. This implies
				that if the space and + flags both appear, the space flag
				will be ignored.
		#		The value is to be converted to an alternate form. For c,
				d, s, and u conversions, the flag has no effect. For o
				conversions, it increases the precision to force the first
				digit of the result to be zero. For x, and X conversions,
				a nonzero result will have 0x or 0X prefixed to it. For
				e, E, f, g, and G conversions, the result will always
				contain a decimal point, even if no digits follow the
				point. For g and G conversion, trailing zeros in the
				fractional part will not be removed from the result.
		0		The 0 flag pads the field with zeros on the left only;
				this applies to d, i, o, u, x, X, e, E, f, g, and G
				conversions. The leading zeros pad the field width, and
				no space padding is performed. If both the 0 and - flags
				appear, the 0 flag is ignored. If a precision is specified
				for d, i, o, u, x, and X conversions, the 0 flag is
				ignored.
	
	The standard conversions are:
	
		d,i		int			Decimal number.
		o		int			Unsigned octal number (w/o a leading zero).
		x,X		int			Unsigned hexadecimal number (w/o a leading 0x
								or oX), using abcdef or ABCDEF for 10,,15.
		u		int			Unsigned decimal number.
		c		int			Single character.
		s		char*			C string.
		f		double		[-]m.dddddd, where the number of d's is given
								by the precision (default 6).
		e,E		double		[-]m.ddddddexx or [-]m.ddddddExx, where the
								number of d's is given by the precision
								(default 6).
		g,G		double		Use %e or %E if the exponent is less than -4
								or greater than or equal to the precision;
								otherwise use %f. Trailing zeros and a
								trailing decimal point are not printed.
		p		void*		Pointer.
		%					No argument is converted; print a %.
	
	The supported MPW extensions are:
	
		P		StringPtr	Pascal string.
	
	Of the MPW printf conversion characters only 'n' is not supported.
	In addition to the standard characters, the following are defined:
	
		b		Boolean			Outputs either true or false.
		j		Point			Used like the d option.
		J		point*			Fixed point. Used like the f option.
		F		Fixed			Used like the f option.
		T		Fract			Used like the f option.
		r		Rect*			Displays the rect with each piece seperated by
								, . Used like the d option.
		R		rectangle*		Similar to r but for fixed point rectangles.
								Used like the f option.
		M		mapping.		Displays the mapping seperated by ,  and
								\n. The third column is displayed as fract.
								Used like the f option.
	
		
	dprintf requires that the MacsBug dcmd dprintf is present.
	
	Characters used:	 bcdefghij l nop rs u  x
							    EFG  J LM  P R T   X
	Not used:			a         k m   q  t vw yz
							ABCD   HI K  NO Q S UVW YZ
	
	See the MPW C Reference for more information on printf.

Echo "															n
	PRINT			OFF,NOHDR									n
	INCLUDE		'Traps.a'										n
	PRINT			ON											n
	PROC														n
	MOVEA.L		SP,A0		; Save the stackPtr					n
	_DebugStr													n
	SUBQ			#4,SP		; Fix the stack					n
	ENDPROC														n
	END															n
" | Asm -l

*********************************************************************/
EXTERN_API_C( void )
dprintf							(StringPtr 				traceoption,
								 char 					format[],
								 ...)														THREEWORDINLINE(0x204F, 0xABFF, 0x594F);

/*********************************************************************

ROUTINE
	void* check_dprintf(assertion, traceOption, format, ...)

DESCRIPTION
	If assertion is non-zero then assertion is returned. Otherwise the
	dprintf is invoked and zero is returned.
	
Echo "															n
	PRINT			OFF,NOHDR									n
	INCLUDE		'Traps.a'										n
	PRINT			ON											n
	PROC														n
	MOVE.L		(SP)+,D0		; Pop value into D0				n
	BNE.S			@1			; If !0 then branch				n
	MOVEA.L		SP,A0			; Save the stackPtr				n
	_DebugStr													n
	SUBQ			#4,SP		; Fix the stack					n
	CLR.L			D0			; Result is zero				n
@1	SUBQ			#4,SP		; Fix stack for pop				n
	ENDPROC														n
	END															n
" | Asm -l
	
*********************************************************************/
#ifdef applec
EXTERN_API_C( void *)
check_dprintf					(void *					assertion,
								 StringPtr 				traceOption,
								 char 					format[],
								 ...)														SEVENWORDINLINE(0x201F, 0x6608, 0x204F, 0xABFF, 0x594F, 0x7000, 0x594F);

#endif  /*  defined(applec)  */

#ifdef applec
EXTERN_API_C( void *)
checkpos_dprintf				(void *					assertion,
								 StringPtr 				traceOption,
								 char 					format[],
								 ...)														SEVENWORDINLINE(0x201F, 0x6C08, 0x204F, 0xABFF, 0x594F, 0x7000, 0x594F);

#endif  /*  defined(applec)  */


#if TARGET_CPU_68K && ! TARGET_RT_MAC_CFM
#define DebugAssert(componentSignature, options, assertionString, exceptionString, errorString, fileName, lineNumber, value)	\
{																												\
	OSType	Signature = componentSignature;																		\
	char 	*SignaturePtr = (char *)(&Signature);																\
	dprintf(notrace, "Assert Failed\nComponent	%c%c%c%c\nAssert	%s\nFile	%s	Line	%d\n",		\
				SignaturePtr[0], SignaturePtr[1], 																\
				SignaturePtr[2], SignaturePtr[3],																\
				assertionString,																				\
				fileName, lineNumber);																			\
}
#else
extern void DebugAssert(	OSType		componentSignature,
							UInt32		options,
							char *		assertionString,
							char *		exceptionString,
							char *		errorString,
							char *		fileName,
							long		lineNumber,
							void *		value);
#endif


/*********************************************************************

MACRO
	check(assertion)

DESCRIPTION
	If debugging is on then check will test assertion and if it fails
	break into the debugger. Otherwise check does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define check(assertion) 												\
	do {																\
		if (assertion) ;												\
		else __DEBUGGERBREAK;											\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define check(assertion) 												\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS, 			\
						QuoteExceptionString(assertion), 				\
						nil, nil, nil, 0, 0); 							\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define check(assertion) 												\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,			\
						QuoteExceptionString(assertion), nil, nil,		\
						__FILE__, __LINE__, 0);	 						\
		}																\
	} while (false)
#else
#define check(assertion)
#endif

/*********************************************************************

MACRO
	ncheck(assertion)

DESCRIPTION
	If debugging is on then ncheck will test !assertion and if it fails
	break into the debugger. Otherwise ncheck does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define ncheck(assertion) 											\
	do {															\
		if (assertion) __DEBUGGERBREAK;	 							\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define ncheck(assertion) 											\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS, 		\
			QuoteExceptionString(assertion), 						\
			nil, nil, nil, 0, __privateAssertion);	 				\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define ncheck(assertion) 											\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 	\
			QuoteExceptionString(assertion), nil, nil,				\
			__FILE__, __LINE__, __privateAssertion);				\
		}															\
	} while (false)
#else
#define ncheck(assertion)
#endif

/*********************************************************************

MACRO
	check_action(assertion, action)

DESCRIPTION
	If debugging is on then check_action will test assertion and if it
	fails break into the debugger then execute action. Otherwise
	check_action does nothing.
	
*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define check_action(assertion, action)  							\
	do {															\
		if (assertion) ;											\
		else {														\
			__DEBUGGERBREAK;										\
			{ action; }												\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define check_action(assertion, action)  							\
	do {															\
		if (assertion) ;											\
		else {														\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS, 		\
						QuoteExceptionString(assertion), 			\
						nil, nil, nil, 0, 0);  						\
			{ action; }												\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define check_action(assertion, action)  							\
	do {															\
		if (assertion) ;											\
		else {														\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,		\
						QuoteExceptionString(assertion), nil, nil,	\
						__FILE__, __LINE__, 0);	 					\
			{ action; }												\
		}															\
	} while (false)
#else
#define check_action(assertion, action)
#endif

/**************************************************************************************

MACRO
	ncheck_action(assertion, action)

DESCRIPTION
	If debugging is on then ncheck_action will test !assertion and if
	it fails break into the debugger then execute action. Otherwise
	ncheck_action does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define ncheck_action(assertion, action)  									\
	do {																	\
		if (assertion) {													\
			__DEBUGGERBREAK;												\
			{ action; }														\
		}																	\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define ncheck_action(assertion, action)  									\
	do {																	\
		void*	__privateAssertion	= (void*)(assertion);	 				\
																			\
		if (__privateAssertion) {											\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 			\
						QuoteExceptionString(assertion), 					\
						nil, nil, nil, 0, __privateAssertion);	 			\
			{ action; }														\
		}																	\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define ncheck_action(assertion, action)  									\
	do {																	\
		void*	__privateAssertion	= (void*)(assertion);	 				\
																			\
		if (__privateAssertion) {											\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,				\
						QuoteExceptionString(assertion), 					\
						nil, nil, __FILE__, __LINE__, __privateAssertion);	\
			{ action; }														\
		}																	\
	} while (false)
#else
#define ncheck_action(assertion, action)
#endif

/*********************************************************************

MACRO
	require(assertion, exception)

DESCRIPTION
	require will test assertion and if it fails:
		break into the debugger if debugging is on.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define require(assertion, exception)  									\
	do {																\
		if (assertion) ;												\
		else {															\
			__DEBUGGERBREAK;											\
			goto exception;												\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define require(assertion, exception)  									\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 		\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception),				\
						 nil, nil, 0, 0);								\
			goto exception;												\
		}																\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define require(assertion, exception)  									\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 		\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception),	 			\
						nil, __FILE__, __LINE__, 0);					\
			goto exception;												\
		}																\
	} while (false)
#else
#define require(assertion, exception)  									\
	do {																\
		if (assertion) ;												\
		else {															\
			goto exception;												\
		}																\
	} while (false)
#endif

/*********************************************************************

MACRO
	nrequire(assertion, exception)

DESCRIPTION
	nrequire will test !assertion and if it fails:
		break into the debugger if debugging is on.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define nrequire(assertion, exception)  							\
	do {															\
		if (assertion) {											\
			__DEBUGGERBREAK;										\
			goto exception;											\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define nrequire(assertion, exception)  							\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);			\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 	\
						QuoteExceptionString(assertion),  			\
						QuoteExceptionString(exception),			\
						nil, nil, 0, __privateAssertion);	 		\
			goto exception;											\
		}															\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define nrequire(assertion, exception)  							\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 	\
						QuoteExceptionString(assertion),			\
						QuoteExceptionString(exception), nil,		\
						__FILE__, __LINE__, __privateAssertion);	\
			goto exception;											\
		}															\
	} while (false)
#else
#define nrequire(assertion, exception)  							\
	do {															\
		if (assertion) {											\
			goto exception;											\
		}															\
	} while (false)
#endif

/*********************************************************************

MACRO
	require_action(assertion, exception, action)

DESCRIPTION
	require_action will test assertion and if it fails:
		break into the debugger if debugging is on.
		execute action.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define require_action(assertion, exception, action)  				\
	do {															\
		if (assertion) ;											\
		else {														\
			__DEBUGGERBREAK;										\
			{ action; }												\
			goto exception;											\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define require_action(assertion, exception, action)				\
	do {															\
		if (assertion) ;											\
		else {														\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 	\
						QuoteExceptionString(assertion),			\
						QuoteExceptionString(exception),			\
						nil, nil, 0, 0);	 						\
			{ action; }												\
			goto exception;											\
		}															\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define require_action(assertion, exception, action)  				\
	do {															\
		if (assertion) ;											\
		else {														\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 	\
						QuoteExceptionString(assertion),  			\
						QuoteExceptionString(exception),			\
						nil, __FILE__, __LINE__, 0);	 			\
			{ action; }												\
			goto exception;											\
		}															\
	} while (false)
#else
#define require_action(assertion, exception, action) 				\
	do {															\
		if (assertion) ;											\
		else {														\
			{ action; }												\
			goto exception;											\
		}															\
	} while (false)
#endif

/*********************************************************************

MACRO
	nrequire_action(assertion, exception, action)

DESCRIPTION
	nrequire_action will test !assertion and if it fails:
		break into the debugger if debugging is on.
		execute action.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define nrequire_action(assertion, exception, action)  				\
	do {															\
		if (assertion) {											\
			__DEBUGGERBREAK;										\
			{ action; }												\
			goto exception;											\
		}															\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define nrequire_action(assertion, exception, action)  				\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 	\
						QuoteExceptionString(assertion),  			\
						QuoteExceptionString(exception),			\
						nil, nil, 0, __privateAssertion);	 		\
			{ action; }												\
			goto exception;											\
		}															\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define nrequire_action(assertion, exception, action)  				\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, DEBUG_NO_OPTIONS,	 	\
						QuoteExceptionString(assertion),  			\
						QuoteExceptionString(exception), nil,		\
						__FILE__, __LINE__, __privateAssertion);	\
			{ action; }												\
			goto exception;											\
		}															\
	} while (false)
#else
#define nrequire_action(assertion, exception, action)  				\
	do {															\
		if (assertion) {											\
			{ action; }												\
			goto exception;											\
		}															\
	} while (false)
#endif


/*********************************************************************

MACRO
	require_quiet(assertion, exception)

DESCRIPTION
	require_quiet will test assertion and if it fails:
		goto exception.

*********************************************************************/

#define require_quiet(assertion, exception)  						\
	do {															\
		if (assertion) ;											\
		else {														\
			goto exception;											\
		}															\
	} while (false)


/*********************************************************************

MACRO
	require_action_quiet(assertion, exception, action)

DESCRIPTION
	require_action_quiet will test assertion and if it fails:
		execute action.
		goto exception.

*********************************************************************/

#define require_action_quiet(assertion, exception, action) 			\
	do {															\
		if (assertion) ;											\
		else {														\
			{ action; }												\
			goto exception;											\
		}															\
	} while (false)



/*********************************************************************

MACRO
	check_option(assertion, options)

DESCRIPTION
	If debugging is on then check will test assertion and if it fails
	break into the debugger. Otherwise check does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define check_option(assertion, options) 								\
	do {																\
		if (assertion) ;												\
		else __DEBUGGERBREAK;											\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define check_option(assertion, options) 								\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options, 					\
						QuoteExceptionString(assertion), 				\
						nil, nil, nil, 0, 0); 							\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define check_option(assertion, options) 								\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options,					\
						QuoteExceptionString(assertion), nil, nil, 		\
						__FILE__, __LINE__, 0);	 						\
		}																\
	} while (false)
#else
#define check_option(assertion, options)
#endif

/*********************************************************************

MACRO
	ncheck_option(assertion, options)

DESCRIPTION
	If debugging is on then ncheck will test !assertion and if it fails
	break into the debugger. Otherwise ncheck does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define ncheck_option(assertion, options) 							\
	do {															\
		if (assertion) __DEBUGGERBREAK;	 							\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define ncheck_option(assertion, options) 							\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, options, 				\
			QuoteExceptionString(assertion), 						\
			nil, nil, nil, 0, __privateAssertion);	 				\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define ncheck_option(assertion, options) 							\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, options,	 			\
			QuoteExceptionString(assertion), nil, nil,				\
			__FILE__, __LINE__, __privateAssertion);				\
		}															\
	} while (false)
#else
#define ncheck_option(assertion, options)
#endif

/*********************************************************************

MACRO
	check_action_option(assertion, action, options)

DESCRIPTION
	If debugging is on then check_action will test assertion and if it
	fails break into the debugger then execute action. Otherwise
	check_action does nothing.
	
*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define check_action_option(assertion, action, options)				\
	do {															\
		if (assertion) ;											\
		else {														\
			__DEBUGGERBREAK;										\
			{ action; }												\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define check_action_option(assertion, action, options)				\
	do {															\
		if (assertion) ;											\
		else {														\
			DebugAssert(COMPONENT_SIGNATURE, options, 				\
						QuoteExceptionString(assertion), 			\
						nil, nil, nil, 0, 0);  						\
			{ action; }												\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define check_action_option(assertion, action, options)				\
	do {															\
		if (assertion) ;											\
		else {														\
			DebugAssert(COMPONENT_SIGNATURE, options,				\
						QuoteExceptionString(assertion), nil,		\
						nil, __FILE__, __LINE__, 0);	 			\
			{ action; }												\
		}															\
	} while (false)
#else
#define check_action_option(assertion, action, options)
#endif

/**************************************************************************************

MACRO
	ncheck_action_option(assertion, action, options)

DESCRIPTION
	If debugging is on then ncheck_action will test !assertion and if
	it fails break into the debugger then execute action. Otherwise
	ncheck_action does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define ncheck_action_option(assertion, action, options)					\
	do {																	\
		if (assertion) {													\
			__DEBUGGERBREAK;												\
			{ action; }														\
		}																	\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define ncheck_action_option(assertion, action, options)					\
	do {																	\
		void*	__privateAssertion	= (void*)(assertion);	 				\
																			\
		if (__privateAssertion) {											\
			DebugAssert(COMPONENT_SIGNATURE, options,	 					\
						QuoteExceptionString(assertion), 					\
						nil, nil, nil, 0, __privateAssertion);	 			\
			{ action; }														\
		}																	\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define ncheck_action_option(assertion, action, options)					\
	do {																	\
		void*	__privateAssertion	= (void*)(assertion);	 				\
																			\
		if (__privateAssertion) {											\
			DebugAssert(COMPONENT_SIGNATURE, options,						\
						QuoteExceptionString(assertion), 					\
						nil, nil, __FILE__, __LINE__, __privateAssertion);	\
			{ action; }														\
		}																	\
	} while (false)
#else
#define ncheck_action_option(assertion, action, options)
#endif

/*********************************************************************

MACRO
	require_option(assertion, exception, options)

DESCRIPTION
	require will test assertion and if it fails:
		break into the debugger if debugging is on.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define require_option(assertion, exception, options)					\
	do {																\
		if (assertion) ;												\
		else {															\
			__DEBUGGERBREAK;											\
			goto exception;												\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define require_option(assertion, exception, options)					\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception), 				\
						nil, nil, 0, 0);								\
			goto exception;												\
		}																\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define require_option(assertion, exception, options)					\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception),	 			\
						nil, __FILE__, __LINE__, 0);					\
			goto exception;												\
		}																\
	} while (false)
#else
#define require_option(assertion, exception, options)					\
	do {																\
		if (assertion) ;												\
		else {															\
			goto exception;												\
		}																\
	} while (false)
#endif

/*********************************************************************

MACRO
	nrequire_option(assertion, exception, options)

DESCRIPTION
	nrequire will test !assertion and if it fails:
		break into the debugger if debugging is on.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define nrequire_option(assertion, exception, options)				\
	do {															\
		if (assertion) {											\
			__DEBUGGERBREAK;										\
			goto exception;											\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define nrequire_option(assertion, exception, options)				\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);			\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, options,	 			\
						QuoteExceptionString(assertion),  			\
						QuoteExceptionString(exception),			\
						nil, nil, 0, __privateAssertion);	 		\
			goto exception;											\
		}															\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define nrequire_option(assertion, exception, options)				\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, options,	 			\
						QuoteExceptionString(assertion),			\
						QuoteExceptionString(exception), nil,		\
						__FILE__, __LINE__, __privateAssertion);	\
			goto exception;											\
		}															\
	} while (false)
#else
#define nrequire_option(assertion, exception, options)				\
	do {															\
		if (assertion) {											\
			goto exception;											\
		}															\
	} while (false)
#endif

/*********************************************************************

MACRO
	require_action_option(assertion, exception, action, options)

DESCRIPTION
	require_action will test assertion and if it fails:
		break into the debugger if debugging is on.
		execute action.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define require_action_option(assertion, exception, action, options)	\
	do {																\
		if (assertion) ;												\
		else {															\
			__DEBUGGERBREAK;											\
			{ action; }													\
			goto exception;												\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define require_action_option(assertion, exception, action, options)	\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),				\
						QuoteExceptionString(exception),				\
						nil, nil, 0, 0);	 							\
			{ action; }													\
			goto exception;												\
		}																\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define require_action_option(assertion, exception, action, options)	\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception), nil,			\
						__FILE__, __LINE__, 0);	 						\
			{ action; }													\
			goto exception;												\
		}																\
	} while (false)
#else
#define require_action_option(assertion, exception, action, options)	\
	do {																\
		if (assertion) ;												\
		else {															\
			{ action; }													\
			goto exception;												\
		}																\
	} while (false)
#endif

/*********************************************************************

MACRO
	nrequire_action_option(assertion, exception, action, options)

DESCRIPTION
	nrequire_action will test !assertion and if it fails:
		break into the debugger if debugging is on.
		execute action.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define nrequire_action_option(assertion, exception, action, options)	\
	do {																\
		if (assertion) {												\
			__DEBUGGERBREAK;											\
			{ action; }													\
			goto exception;												\
		}																\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define nrequire_action_option(assertion, exception, action, options)	\
	do {																\
		void*	__privateAssertion	= (void*)(assertion);	 			\
																		\
		if (__privateAssertion) {										\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception),				\
						nil, nil, 0, __privateAssertion);	 			\
			{ action; }													\
			goto exception;												\
		}																\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define nrequire_action_option(assertion, exception, action, options)	\
	do {																\
		void*	__privateAssertion	= (void*)(assertion);	 			\
																		\
		if (__privateAssertion) {										\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception), nil,			\
						__FILE__, __LINE__, __privateAssertion);		\
			{ action; }													\
			goto exception;												\
		}																\
	} while (false)
#else
#define nrequire_action_option(assertion, exception, action, options)	\
	do {																\
		if (assertion) {												\
			{ action; }													\
			goto exception;												\
		}																\
	} while (false)
#endif



/*********************************************************************

MACRO
	check_string(assertion, options, errorString)

DESCRIPTION
	If debugging is on then check will test assertion and if it fails
	break into the debugger. Otherwise check does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define check_string(assertion, options, errorString) 					\
	do {																\
		if (assertion) ;												\
		else __DEBUGGERBREAK;											\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define check_string(assertion, options, errorString) 					\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options, 					\
						QuoteExceptionString(assertion), 				\
						nil, errorString, nil, 0, 0); 					\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define check_string(assertion, options, errorString) 					\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options,					\
						QuoteExceptionString(assertion), nil, 			\
						errorString, __FILE__, __LINE__, 0);	 		\
		}																\
	} while (false)
#else
#define check_string(assertion, options, errorString)
#endif

/*********************************************************************

MACRO
	ncheck_string(assertion, options, errorString)

DESCRIPTION
	If debugging is on then ncheck will test !assertion and if it fails
	break into the debugger. Otherwise ncheck does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define ncheck_string(assertion, options, errorString) 				\
	do {															\
		if (assertion) __DEBUGGERBREAK;	 							\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define ncheck_string(assertion, options, errorString) 				\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, options, 				\
			QuoteExceptionString(assertion), 						\
			nil, errorString, nil, 0, __privateAssertion);	 		\
		}															\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define ncheck_string(assertion, options, errorString) 				\
	do {															\
		void*	__privateAssertion	= (void*)(assertion);	 		\
																	\
		if (__privateAssertion) {									\
			DebugAssert(COMPONENT_SIGNATURE, options,	 			\
			QuoteExceptionString(assertion), nil, 					\
			errorString, __FILE__, __LINE__, __privateAssertion);	\
		}															\
	} while (false)
#else
#define ncheck_string(assertion, options, errorString)
#endif

/*********************************************************************

MACRO
	check_action_string(assertion, action, options, errorString)

DESCRIPTION
	If debugging is on then check_action will test assertion and if it
	fails break into the debugger then execute action. Otherwise
	check_action does nothing.
	
*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define check_action_string(assertion, action, options, errorString)	\
	do {																\
		if (assertion) ;												\
		else {															\
			__DEBUGGERBREAK;											\
			{ action; }													\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define check_action_string(assertion, action, options, errorString)	\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options, 					\
						QuoteExceptionString(assertion), 				\
						nil, errorString, nil, 0, 0);  					\
			{ action; }													\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define check_action_string(assertion, action, options, errorString)	\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options,					\
						QuoteExceptionString(assertion), nil,			\
						errorString, __FILE__, __LINE__, 0);	 		\
			{ action; }													\
		}																\
	} while (false)
#else
#define check_action_string(assertion, action, options, errorString)
#endif

/**************************************************************************************

MACRO
	ncheck_action_string(assertion, action, options, errorString)

DESCRIPTION
	If debugging is on then ncheck_action will test !assertion and if
	it fails break into the debugger then execute action. Otherwise
	ncheck_action does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define ncheck_action_string(assertion, action, options, errorString)		\
	do {																	\
		if (assertion) {													\
			__DEBUGGERBREAK;												\
			{ action; }														\
		}																	\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define ncheck_action_string(assertion, action, options, errorString)		\
	do {																	\
		void*	__privateAssertion	= (void*)(assertion);	 				\
																			\
		if (__privateAssertion) {											\
			DebugAssert(COMPONENT_SIGNATURE, options,	 					\
						QuoteExceptionString(assertion), 					\
						nil, errorString, nil, 0, __privateAssertion);	 	\
			{ action; }														\
		}																	\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define ncheck_action_string(assertion, action, options, errorString)		\
	do {																	\
		void*	__privateAssertion	= (void*)(assertion);	 				\
																			\
		if (__privateAssertion) {											\
			DebugAssert(COMPONENT_SIGNATURE, options,						\
						QuoteExceptionString(assertion), nil, errorString,	\
						__FILE__, __LINE__, __privateAssertion);	 		\
			{ action; }														\
		}																	\
	} while (false)
#else
#define ncheck_action_string(assertion, action, options, errorString)
#endif

/*********************************************************************

MACRO
	require_string(assertion, exception, options, errorString)

DESCRIPTION
	require will test assertion and if it fails:
		break into the debugger if debugging is on.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define require_string(assertion, exception, options, errorString)		\
	do {																\
		if (assertion) ;												\
		else {															\
			__DEBUGGERBREAK;											\
			goto exception;												\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define require_string(assertion, exception, options, errorString)		\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception), 				\
						errorString, nil, 0, 0);						\
			goto exception;												\
		}																\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define require_string(assertion, exception, options, errorString)		\
	do {																\
		if (assertion) ;												\
		else {															\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception),	 			\
						errorString, __FILE__, __LINE__, 0);			\
			goto exception;												\
		}																\
	} while (false)
#else
#define require_string(assertion, exception, options, errorString)		\
	do {																\
		if (assertion) ;												\
		else {															\
			goto exception;												\
		}																\
	} while (false)
#endif

/*********************************************************************

MACRO
	nrequire_string(assertion, exception, options, errorString)

DESCRIPTION
	nrequire will test !assertion and if it fails:
		break into the debugger if debugging is on.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define nrequire_string(assertion, exception, options, errorString)		\
	do {																\
		if (assertion) {												\
			__DEBUGGERBREAK;											\
			goto exception;												\
		}																\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define nrequire_string(assertion, exception, options, errorString)		\
	do {																\
		void*	__privateAssertion	= (void*)(assertion);				\
																		\
		if (__privateAssertion) {										\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),  				\
						QuoteExceptionString(exception),				\
						errorString, nil, 0, __privateAssertion);	 	\
			goto exception;												\
		}																\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define nrequire_string(assertion, exception, options, errorString)		\
	do {																\
		void*	__privateAssertion	= (void*)(assertion);	 			\
																		\
		if (__privateAssertion) {										\
			DebugAssert(COMPONENT_SIGNATURE, options,	 				\
						QuoteExceptionString(assertion),				\
						QuoteExceptionString(exception), errorString,	\
						 __FILE__, __LINE__, __privateAssertion);		\
			goto exception;												\
		}																\
	} while (false)
#else
#define nrequire_string(assertion, exception, options, errorString)		\
	do {																\
		if (assertion) {												\
			goto exception;												\
		}																\
	} while (false)
#endif

/*********************************************************************

MACRO
	require_action_string(assertion, exception, action, options, errorString)

DESCRIPTION
	require_action will test assertion and if it fails:
		break into the debugger if debugging is on.
		execute action.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define require_action_string(assertion, exception, action, options, errorString)	\
	do {																			\
		if (assertion) ;															\
		else {																		\
			__DEBUGGERBREAK;														\
			{ action; }																\
			goto exception;															\
		}																			\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define require_action_string(assertion, exception, action, options, errorString)	\
	do {																			\
		if (assertion) ;															\
		else {																		\
			DebugAssert(COMPONENT_SIGNATURE, options,	 							\
						QuoteExceptionString(assertion),							\
						QuoteExceptionString(exception),							\
						errorString, nil, 0, 0);	 								\
			{ action; }																\
			goto exception;															\
		}																			\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define require_action_string(assertion, exception, action, options, errorString)	\
	do {																			\
		if (assertion) ;															\
		else {																		\
			DebugAssert(COMPONENT_SIGNATURE, options,	 							\
						QuoteExceptionString(assertion),  							\
						QuoteExceptionString(exception),							\
						errorString, __FILE__, __LINE__, 0);	 					\
			{ action; }																\
			goto exception;															\
		}																			\
	} while (false)
#else
#define require_action_string(assertion, exception, action, options, errorString)	\
	do {																			\
		if (assertion) ;															\
		else {																		\
			{ action; }																\
			goto exception;															\
		}																			\
	} while (false)
#endif

/*********************************************************************

MACRO
	nrequire_action_string(assertion, exception, action, options, errorString)

DESCRIPTION
	nrequire_action will test !assertion and if it fails:
		break into the debugger if debugging is on.
		execute action.
		goto exception.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define nrequire_action_string(assertion, exception, action, options, errorString)	\
	do {																			\
		if (assertion) {															\
			__DEBUGGERBREAK;														\
			{ action; }																\
			goto exception;															\
		}																			\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define nrequire_action_string(assertion, exception, action, options, errorString)	\
	do {																			\
		void*	__privateAssertion	= (void*)(assertion);					 		\
																					\
		if (__privateAssertion) {													\
			DebugAssert(COMPONENT_SIGNATURE, options,	 							\
						QuoteExceptionString(assertion),  							\
						QuoteExceptionString(exception),							\
						errorString, nil, 0, __privateAssertion);	 				\
			{ action; }																\
			goto exception;															\
		}																			\
	} while (false)
#elif DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define nrequire_action_string(assertion, exception, action, options, errorString)	\
	do {																			\
		void*	__privateAssertion	= (void*)(assertion);	 						\
																					\
		if (__privateAssertion) {													\
			DebugAssert(COMPONENT_SIGNATURE, options,	 							\
						QuoteExceptionString(assertion),  							\
						QuoteExceptionString(exception), errorString,				\
						__FILE__, __LINE__, __privateAssertion);					\
			{ action; }																\
			goto exception;															\
		}																			\
	} while (false)
#else
#define nrequire_action_string(assertion, exception, action, options, errorString)	\
	do {																			\
		if (assertion) {															\
			{ action; }																\
			goto exception;															\
		}																			\
	} while (false)
#endif


/*********************************************************************

MACRO
	debug_string(assertion, options, errorString)

DESCRIPTION
	If debugging is on then debug_string will assert and
	break into the debugger. Otherwise check does nothing.

*********************************************************************/

#if	__DEBUG_BREAK_ONLY
#define debug_string(options, errorString) 								\
	do {																\
		__DEBUGGERBREAK;												\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_EXTERNAL
#define debug_string(options, errorString) 								\
	do {																\
		DebugAssert(COMPONENT_SIGNATURE, options, 						\
					"debug_string", 									\
					nil, errorString, nil, 0, 0); 						\
	} while (false)
#elif	DEBUGLEVEL == DEBUG_LEVEL_INTERNAL
#define debug_string(options, errorString) 								\
	do {																\
		DebugAssert(COMPONENT_SIGNATURE, options,						\
					"debug_string", nil, 								\
					errorString, __FILE__, __LINE__, 0);	 			\
	} while (false)
#else
#define debug_string(options, errorString)
#endif





#if PRAGMA_STRUCT_ALIGN
	#pragma options align=reset
#elif PRAGMA_STRUCT_PACKPUSH
	#pragma pack(pop)
#elif PRAGMA_STRUCT_PACK
	#pragma pack()
#endif

#ifdef PRAGMA_IMPORT_OFF
#pragma import off
#elif PRAGMA_IMPORT
#pragma import reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __DEBUGASSERT__ */

